# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: getprefix.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import re, upcean.validate, upcean.convert;

def get_upca_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(len(upc)==8):
  upc = upcean.convert.convert_barcode_from_upce_to_upca(upc);
 if(re.findall("^0(\d{13})", upc)):
  upc_matches = re.findall("^0(\d{13})", upc);
  upc = upc_matches[0];
 if(re.findall("^0(\d{12})", upc)):
  upc_matches = re.findall("^0(\d{12})", upc);
  upc = upc_matches[0];
 if(not re.findall("^(\d{12})", upc)):
  return False;
 upc_matches = re.findall("^(\d{1})(\d{5})(\d{5})(\d{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'packagecode': None, 'numbersystem': pre_upc_type[0], 'manufacturer': pre_upc_type[1], 'product': pre_upc_type[2], 'checkdigit': pre_upc_type[3]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_upca_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_upca_barcode_manufacturer(upc):
 upc = str(upc);
 product = get_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("manufacturer", False);
def get_upca_barcode_product(upc):
 upc = str(upc);
 product = get_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("product", False);
def get_upca_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_upca_barcode_info_from_upce(upc):
 return get_upca_barcode_info(upcean.convert.convert_barcode_from_upce_to_upca(upc));
def get_upce_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(re.findall("^0(\d{13})", upc)):
  upc_matches = re.findall("^0(\d{13})", upc);
  upc = upc_matches[0];
 if(re.findall("^0(\d{12})", upc)):
  upc_matches = re.findall("^0(\d{12})", upc);
  upc = upc_matches[0];
 if(len(upc)==12):
  upc = upcean.convert.convert_barcode_from_upca_to_upce(upc);
 if(not re.findall("^(\d{8})", upc)):
  return False;
 get_ns = None;
 get_manufac = None;
 get_product = None;
 get_checksum = None;
 if(re.findall("(0|1)(\d{2})(\d{3})(0)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{2})(\d{3})(0)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2]+upc_matches[3];
  get_checksum = upc_matches[4];
 if(re.findall("(0|1)(\d{2})(\d{3})(1)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{2})(\d{3})(1)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2]+upc_matches[3];
  get_checksum = upc_matches[4];
 if(re.findall("(0|1)(\d{2})(\d{3})(2)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{2})(\d{3})(2)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2]+upc_matches[3];
  get_checksum = upc_matches[4];
 if(re.findall("(0|1)(\d{3})(\d{2})(3)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{3})(\d{2})(3)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2]+upc_matches[3];
  get_checksum = upc_matches[4];
 if(re.findall("(0|1)(\d{4})(\d{1})(4)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{4})(\d{1})(4)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2]+upc_matches[3];
  get_checksum = upc_matches[4];
 if(re.findall("(0|1)(\d{5})(5)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{5})(5)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2];
  get_checksum = upc_matches[3];
 if(re.findall("(0|1)(\d{5})(6)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{5})(6)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2];
  get_checksum = upc_matches[3];
 if(re.findall("(0|1)(\d{5})(7)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{5})(7)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2];
  get_checksum = upc_matches[3];
 if(re.findall("(0|1)(\d{5})(8)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{5})(8)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2];
  get_checksum = upc_matches[3];
 if(re.findall("(0|1)(\d{5})(9)(\d{1})", upc)):
  upc_matches = re.findall("(0|1)(\d{5})(9)(\d{1})", upc);
  upc_matches = upc_matches[0];
  get_ns = upc_matches[0];
  get_manufac = upc_matches[1];
  get_product = upc_matches[2];
  get_checksum = upc_matches[3];
 upc_type = {'packagecode': None, 'numbersystem': get_ns, 'manufacturer': get_manufac, 'product': get_product, 'checkdigit': get_checksum};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_upce_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_upce_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_upce_barcode_manufacturer(upc):
 upc = str(upc);
 product = get_upce_barcode_info(upc);
 if(not product):
  return False;
 return product.get("manufacturer", False);
def get_upce_barcode_product(upc):
 upc = str(upc);
 product = get_upce_barcode_info(upc);
 if(not product):
  return False;
 return product.get("product", False);
def get_upce_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_upce_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_ean8_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(not re.findall("^(\d{8})", upc)):
  return False;
 upc_matches = re.findall("^(\d{2})(\d{5})(\d{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'packagecode': None, 'numbersystem': pre_upc_type[0], 'manufacturer': None, 'product': pre_upc_type[1], 'checkdigit': pre_upc_type[2]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_ean8_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_ean8_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_ean8_barcode_manufacturer(upc):
 upc = str(upc);
 product = get_ean8_barcode_info(upc);
 if(not product):
  return False;
 return product.get("manufacturer", False);
def get_ean8_barcode_product(upc):
 upc = str(upc);
 product = get_ean8_barcode_info(upc);
 if(not product):
  return False;
 return product.get("product", False);
def get_ean8_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_ean8_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_ean13_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(len(upc)==8):
  upc = upcean.convert.convert_barcode_from_upce_to_upca(upc);
 if(len(upc)==12):
  upc = "0"+upc;
 if(not re.findall("^(\d{13})", upc)):
  return False;
 upc_matches = re.findall("^(\d{2})(\d{5})(\d{5})(\d{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'packagecode': None, 'numbersystem': pre_upc_type[0], 'manufacturer': pre_upc_type[1], 'product': pre_upc_type[2], 'checkdigit': pre_upc_type[3]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_ean13_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_ean13_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_ean13_barcode_manufacturer(upc):
 upc = str(upc);
 product = get_ean13_barcode_info(upc);
 if(not product):
  return False;
 return product.get("manufacturer", False);
def get_ean13_barcode_product(upc):
 upc = str(upc);
 product = get_ean13_barcode_info(upc);
 if(not product):
  return False;
 return product.get("product", False);
def get_ean13_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_ean13_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_itf14_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(len(upc)==12):
  upc = "00"+upc;
 if(len(upc)==13):
  upc = "0"+upc;
 if(not re.findall("^(\d{14})", upc)):
  return False;
 upc_matches = re.findall("^(\d{1})(\d{2})(\d{5})(\d{5})(\d{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'packagecode': pre_upc_type[0], 'numbersystem': pre_upc_type[1], 'manufacturer': pre_upc_type[2], 'product': pre_upc_type[3], 'checkdigit': pre_upc_type[4]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_itf14_barcode_packagecode(upc):
 upc = str(upc);
 product = get_itf14_barcode_info(upc);
 if(not product):
  return False;
 return product.get("packagecode", False);
def get_itf14_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_itf14_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_itf14_barcode_manufacturer(upc):
 upc = str(upc);
 product = get_itf14_barcode_info(upc);
 if(not product):
  return False;
 return product.get("manufacturer", False);
def get_itf14_barcode_product(upc):
 upc = str(upc);
 product = get_itf14_barcode_info(upc);
 if(not product):
  return False;
 return product.get("product", False);
def get_itf14_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_itf14_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
'''
// Get Number System Prefix for UPC-A barcodes
// Source: http://www.morovia.com/education/symbology/upc-a.asp
// Source: http://www.computalabel.com/aboutupc.htm
'''
def get_upca_barcode_ns(upc):
 upc = str(upc);
 if(re.findall("^0(\d{12})", upc)):
  upc_matches = re.findall("^0(\d{12})", upc);
  upc = upc_matches[0];
 if(not re.findall("^(\d{12})", upc)):
  return False;
 if(re.findall("^(0)", upc)):
  return "Regular UPC";
 if(re.findall("^(1)", upc)):
  return "Regular UPC";
 if(re.findall("^(2)", upc)):
  return "Variable Weight Items";
 if(re.findall("^(3)", upc)):
  return "DrugHealth Items";
 if(re.findall("^(4)", upc)):
  return "In-store use";
 if(re.findall("^(5)", upc)):
  return "Coupons";
 if(re.findall("^(6)", upc)):
  return "Regular UPC";
 if(re.findall("^(7)", upc)):
  return "Regular UPC";
 if(re.findall("^(8)", upc)):
  return "Regular UPC";
 if(re.findall("^(9)", upc)):
  return "Coupons";
 return False;

'''
// Get ITF-14 Packaging Indicator
// Source: http://www.mecsw.com/specs/itf_14.html
// Source: http://www.qed.org/RBTL/chapters/ch3.3.htm
'''
def get_itf14_barcode_type(upc):
 upc = str(upc);
 if(not re.findall("^(\d{14})", upc)):
  return False;
 if(re.findall("^(0)", upc)):
  return "UPC code of contents differs from case code";
 if(re.findall("^(1)", upc)):
  return "More than each and below inner packs";
 if(re.findall("^(2)", upc)):
  return "More than each and below inner packs";
 if(re.findall("^(3)", upc)):
  return "Inner packs";
 if(re.findall("^(4)", upc)):
  return "Inner packs";
 if(re.findall("^(5)", upc)):
  return "Shipping containers (cartons)";
 if(re.findall("^(6)", upc)):
  return "Shipping containers (cartons)";
 if(re.findall("^(7)", upc)):
  return "Pallet";
 if(re.findall("^(8)", upc)):
  return "Reserved";
 if(re.findall("^(9)", upc)):
  return "Variable quantity content";
 return False;

'''
// Get Goodwill UPC Info.
'''
def get_goodwill_upca_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(re.findall("^0(\d{12})", upc)):
  upc_matches = re.findall("^0(\d{12})", upc);
  upc = upc_matches[0];
 if(not re.findall("^(\d{12})", upc)):
  return False;
 if(not re.findall("^4(\d{11})", upc)):
  return False;
 upc_matches = re.findall("^4(\d{5})(\d{5})(\d{1})", upc);
 upc_matches = upc_matches[0];
 gw_item_type = None;
 if(re.findall("^(4111)", upc)):
  gw_item_type = "Softlines";
 elif(re.findall("^(4666)", upc)):
  gw_item_type = "Hardlines";
 elif(re.findall("^(4555)", upc)):
  gw_item_type = "Shoes/Purses";
 elif(re.findall("^(4190)", upc)):
  gw_item_type = "Target";
 elif(re.findall("^(4230)", upc)):
  gw_item_type = "Jacobs";
 elif(re.findall("^(4333330)", upc)):
  gw_item_type = "Furniture";
 elif(re.findall("^(4120120)", upc)):
  gw_item_type = "Books";
 elif(re.findall("^(4002000)", upc)):
  gw_item_type = "Mystery Dozen Deal";
 elif(re.findall("^(4010000)", upc)):
  gw_item_type = "Bagged Hardlines";
 else:
  gw_item_type = None;
 gw_item_color = None;
 if(re.findall("^(4)(\d{3})(22)", upc)):
  gw_item_color = "Pink";
 elif(re.findall("^(4)(\d{3})(33)", upc)):
  gw_item_color = "Yellow";
 elif(re.findall("^(4)(\d{3})(44)", upc)):
  gw_item_color = "Green";
 elif(re.findall("^(4)(\d{3})(55)", upc)):
  gw_item_color = "Blue";
 elif(re.findall("^(4)(\d{3})(77)", upc)):
  gw_item_color = "Orange";
 else:
  gw_item_color = None;
 price_matches = re.findall("^(\d{3})(\d{2})", upc_matches[1]);
 price_matches = price_matches[0];
 if(price_matches[0]=="399" and gw_item_type=="Mystery Dozen Deal"):
  gw_item_type = "Mystery DVD Deal";
 elif(price_matches[0]=="699" and gw_item_type=="Mystery Dozen Deal"):
  gw_item_type = "Mystery 1/2 Dozen Deal";
 elif(price_matches[0]=="999" and gw_item_type=="Mystery Dozen Deal"):
  gw_item_type = "Mystery Dozen Deal";
 else:
  gw_item_type = gw_item_type;
 price_alt = str(price_matches[0].lstrip('0'))+price_matches[1];
 formated_price = price_matches[0]+"."+price_matches[1];
 formated_price_alt = str(price_matches[0].lstrip('0'))+"."+price_matches[1];
 product = {'numbersystem': str(4), 'code': upc_matches[0], 'price': upc_matches[1], 'pricendnz': price_alt, 'pricewdwz': formated_price, 'pricewdnz': formated_price_alt, 'type': gw_item_type, 'tagcolor': gw_item_color, 'checkdigit': upc_matches[2]};
 if(infotype is None):
  return product;
 if(infotype is not None):
  return product.get(infotype, product);
def get_goodwill_upca_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_goodwill_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_goodwill_upca_barcode_code(upc):
 upc = str(upc);
 product = get_goodwill_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("code", False);
def get_goodwill_upca_barcode_price(upc):
 upc = str(upc);
 product = get_goodwill_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("price", False);
def get_goodwill_upca_barcode_type(upc):
 upc = str(upc);
 product = get_goodwill_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("type", False);
def get_goodwill_upca_barcode_tagcolor(upc):
 upc = str(upc);
 product = get_goodwill_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("tagcolor", False);
def get_goodwill_upca_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_goodwill_upca_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);

'''
// Get variable weight info from UPC-A
// Source: http://wiki.answers.com/Q/How_does_a_price_embedded_bar_code_work
// Source: http://en.wikipedia.org/wiki/Universal_Product_Code#Prefixes
'''
def get_upca_vw_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(re.findall("^0(\d{12})", upc)):
  upc_matches = re.findall("^0(\d{12})", upc);
  upc = upc_matches[0];
 if(not re.findall("^(\d{12})", upc)):
  return False;
 if(not re.findall("^2(\d{11})", upc)):
  return False;
 upc_matches = re.findall("^2(\d{5})(\d{1})(\d{4})(\d{1})", upc);
 upc_matches = upc_matches[0];
 product = {'numbersystem': str(2), 'code': upc_matches[0], 'pricecs': upc_matches[1], 'price': upc_matches[2], 'checkdigit': upc_matches[3]};
 if(infotype is None):
  return product;
 if(infotype is not None):
  return product.get(infotype, product);
def get_vw_barcode_info(upc, infotype=None):
 return get_upca_vw_barcode_info(upc, infotype);
def get_upca_vw_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_upca_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_vw_barcode_numbersystem(upc):
 return get_upca_vw_barcode_numbersystem(upc);
def get_upca_vw_barcode_code(upc):
 upc = str(upc);
 product = get_upca_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("code", False);
def get_vw_barcode_code(upc):
 return get_upca_vw_barcode_code(upc);
def get_upca_vw_barcode_price(upc):
 upc = str(upc);
 product = get_upca_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("price", False);
def get_vw_barcode_price(upc):
 return get_upca_vw_barcode_price(upc);
def get_upca_vw_barcode_pricecs(upc):
 upc = str(upc);
 product = get_upca_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("pricecs", False);
def get_vw_barcode_pricecs(upc):
 return get_upca_vw_barcode_pricecs(upc);
def get_upca_vw_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_upca_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_upca_vw_barcode_checksum(upc):
 return get_upca_vw_barcode_checkdigit(upc);
def get_vw_barcode_checkdigit(upc):
 return get_upca_vw_barcode_checkdigit(upc);
def get_vw_barcode_checksum(upc):
 return get_upca_vw_barcode_checkdigit(upc);

'''
// Get variable weight info from EAN-13
// Source: https://softmatic.com/barcode-ean-13.html#ean-country
'''
def get_ean13_vw_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(re.findall("^0(\d{13})", upc)):
  upc_matches = re.findall("^0(\d{13})", upc);
  upc = upc_matches[0];
 if(not re.findall("^(\d{13})", upc)):
  return False;
 if(not re.findall("^2(\d{12})", upc)):
  return False;
 upc_matches = re.findall("^2(\d{1})(\d{5})(\d{5})(\d{1})", upc);
 upc_matches = upc_matches[0];
 product = {'numbersystem': str(2), 'subnumbersystem': upc_matches[0], 'code': upc_matches[1], 'price': upc_matches[2], 'checkdigit': upc_matches[3]};
 if(infotype is None):
  return product;
 if(infotype is not None):
  return product.get(infotype, product);
def get_ean13_vw_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_ean13_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_ean13_vw_barcode_subnumbersystem(upc):
 upc = str(upc);
 product = get_ean13_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("subnumbersystem", False);
def get_ean13_vw_barcode_code(upc):
 upc = str(upc);
 product = get_ean13_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("code", False);
def get_ean13_vw_barcode_price(upc):
 upc = str(upc);
 product = get_ean13_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("price", False);
def get_ean13_vw_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_ean13_vw_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_ean13_vw_barcode_checksum(upc):
 return get_ean13_vw_barcode_checkdigit(upc);

'''
// Get coupon info
// Source: http://divagirlusa-ivil.tripod.com/austinitecouponers/id29.html
'''
def get_upca_coupon_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(re.findall("^0(\d{12})", upc)):
  upc_matches = re.findall("^0(\d{12})", upc);
  upc = upc_matches[0];
 if(not re.findall("^(\d{12})", upc)):
  return False;
 if(not re.findall("^(5|9)(\d{11})", upc)):
  return False;
 upc_matches = re.findall("^(5|9)(\d{5})(\d{3})(\d{2})(\d{1})", upc);
 upc_matches = upc_matches[0];
 product = {'numbersystem': upc_matches[0], 'manufacturer': upc_matches[1], 'family': upc_matches[2], 'value': upc_matches[3], 'checkdigit': upc_matches[4]};
 if(infotype is None):
  return product;
 if(infotype is not None):
  return product.get(infotype, product);
def get_upca_coupon_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_upca_coupon_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_upca_coupon_barcode_manufacturer(upc):
 upc = str(upc);
 product = get_upca_coupon_barcode_info(upc);
 if(not product):
  return False;
 return product.get("manufacturer", False);
def get_upca_coupon_barcode_family(upc):
 upc = str(upc);
 product = get_upca_coupon_barcode_info(upc);
 if(not product):
  return False;
 return product.get("family", False);
def get_upca_coupon_barcode_value(upc):
 upc = str(upc);
 product = get_upca_coupon_barcode_info(upc);
 if(not product):
  return False;
 return product.get("value", False);
def get_upca_coupon_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_upca_coupon_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_upca_coupon_barcode_value_code(vcode):
 vcode = str(vcode);
 if(re.findall("^(00)", vcode)):
  return "Manual Input Required";
 if(re.findall("^(01)", vcode)):
  return "Free Item";
 if(re.findall("^(02)", vcode)):
  return "Buy 4 Get 1 Free";
 if(re.findall("^(03)", vcode)):
  return "$1.10";
 if(re.findall("^(04)", vcode)):
  return "$1.35";
 if(re.findall("^(05)", vcode)):
  return "$1.40";
 if(re.findall("^(06)", vcode)):
  return "$1.60";
 if(re.findall("^(07)", vcode)):
  return "Buy 3 For $1.50";
 if(re.findall("^(08)", vcode)):
  return "Buy 2 For $3.00";
 if(re.findall("^(09)", vcode)):
  return "Buy 3 For $2.00";
 if(re.findall("^(10)", vcode)):
  return "$0.10";
 if(re.findall("^(11)", vcode)):
  return "$1.85";
 if(re.findall("^(12)", vcode)):
  return "$0.12";
 if(re.findall("^(13)", vcode)):
  return "Buy 4 For $1.00";
 if(re.findall("^(14)", vcode)):
  return "Buy 1 Get 1 Free";
 if(re.findall("^(15)", vcode)):
  return "$0.15";
 if(re.findall("^(16)", vcode)):
  return "Buy 2 Get 1 Free";
 if(re.findall("^(17)", vcode)):
  return "Reserved for future use";
 if(re.findall("^(18)", vcode)):
  return "$2.60";
 if(re.findall("^(19)", vcode)):
  return "Buy 3 Get 1 Free";
 if(re.findall("^(20)", vcode)):
  return "$0.20";
 if(re.findall("^(21)", vcode)):
  return "Buy 2 For $0.35";
 if(re.findall("^(22)", vcode)):
  return "Buy 2 For $0.40";
 if(re.findall("^(23)", vcode)):
  return "Buy 2 For $0.45";
 if(re.findall("^(24)", vcode)):
  return "Buy 2 For $0.50";
 if(re.findall("^(25)", vcode)):
  return "$0.25";
 if(re.findall("^(26)", vcode)):
  return "$2.85";
 if(re.findall("^(27)", vcode)):
  return "Reserved for future use";
 if(re.findall("^(28)", vcode)):
  return "Buy 2 For $0.55";
 if(re.findall("^(29)", vcode)):
  return "$0.29";
 if(re.findall("^(30)", vcode)):
  return "$0.30";
 if(re.findall("^(31)", vcode)):
  return "Buy 2 For $0.60";
 if(re.findall("^(32)", vcode)):
  return "Buy 2 For $0.75";
 if(re.findall("^(33)", vcode)):
  return "Buy 2 For $1.00";
 if(re.findall("^(34)", vcode)):
  return "Buy 2 For $1.25";
 if(re.findall("^(35)", vcode)):
  return "$0.35";
 if(re.findall("^(36)", vcode)):
  return "Buy 2 For $1.50";
 if(re.findall("^(37)", vcode)):
  return "Buy 3 For $0.25";
 if(re.findall("^(38)", vcode)):
  return "Buy 3 For $0.30";
 if(re.findall("^(39)", vcode)):
  return "$0.39";
 if(re.findall("^(40)", vcode)):
  return "$0.40";
 if(re.findall("^(41)", vcode)):
  return "Buy 3 For $0.50";
 if(re.findall("^(42)", vcode)):
  return "Buy 3 For $1.00";
 if(re.findall("^(43)", vcode)):
  return "Buy 2 For $1.10";
 if(re.findall("^(44)", vcode)):
  return "Buy 2 For $1.35";
 if(re.findall("^(45)", vcode)):
  return "$0.45";
 if(re.findall("^(46)", vcode)):
  return "Buy 2 For $1.60";
 if(re.findall("^(47)", vcode)):
  return "Buy 2 For $1.75";
 if(re.findall("^(48)", vcode)):
  return "Buy 2 For $1.85";
 if(re.findall("^(49)", vcode)):
  return "$0.49";
 if(re.findall("^(50)", vcode)):
  return "$0.50";
 if(re.findall("^(51)", vcode)):
  return "Buy 2 For $2.00";
 if(re.findall("^(52)", vcode)):
  return "Buy 3 For $0.55";
 if(re.findall("^(53)", vcode)):
  return "Buy 2 For $0.10";
 if(re.findall("^(54)", vcode)):
  return "Buy 2 For $0.15";
 if(re.findall("^(55)", vcode)):
  return "$0.55";
 if(re.findall("^(56)", vcode)):
  return "Buy 2 For $0.20";
 if(re.findall("^(57)", vcode)):
  return "Buy 2 For $0.25";
 if(re.findall("^(58)", vcode)):
  return "Buy 2 For $0.30";
 if(re.findall("^(59)", vcode)):
  return "$0.59";
 if(re.findall("^(60)", vcode)):
  return "$0.60";
 if(re.findall("^(61)", vcode)):
  return "$10.00";
 if(re.findall("^(62)", vcode)):
  return "$9.50";
 if(re.findall("^(63)", vcode)):
  return "$9.00";
 if(re.findall("^(64)", vcode)):
  return "$8.50";
 if(re.findall("^(65)", vcode)):
  return "$0.65";
 if(re.findall("^(66)", vcode)):
  return "$8.00";
 if(re.findall("^(67)", vcode)):
  return "$7.50";
 if(re.findall("^(68)", vcode)):
  return "$7.00";
 if(re.findall("^(69)", vcode)):
  return "$0.69";
 if(re.findall("^(70)", vcode)):
  return "$0.70";
 if(re.findall("^(71)", vcode)):
  return "$6.50";
 if(re.findall("^(72)", vcode)):
  return "$6.00";
 if(re.findall("^(73)", vcode)):
  return "$5.50";
 if(re.findall("^(74)", vcode)):
  return "$5.00";
 if(re.findall("^(75)", vcode)):
  return "$0.75";
 if(re.findall("^(76)", vcode)):
  return "$1.00";
 if(re.findall("^(77)", vcode)):
  return "$1.25";
 if(re.findall("^(78)", vcode)):
  return "$1.50";
 if(re.findall("^(79)", vcode)):
  return "$0.79";
 if(re.findall("^(80)", vcode)):
  return "$0.80";
 if(re.findall("^(81)", vcode)):
  return "$1.75";
 if(re.findall("^(82)", vcode)):
  return "$2.00";
 if(re.findall("^(83)", vcode)):
  return "$2.25";
 if(re.findall("^(84)", vcode)):
  return "$2.50";
 if(re.findall("^(85)", vcode)):
  return "$0.85";
 if(re.findall("^(86)", vcode)):
  return "$2.75";
 if(re.findall("^(87)", vcode)):
  return "$3.00";
 if(re.findall("^(88)", vcode)):
  return "$3.25";
 if(re.findall("^(89)", vcode)):
  return "$0.89";
 if(re.findall("^(90)", vcode)):
  return "$0.90";
 if(re.findall("^(91)", vcode)):
  return "$3.50";
 if(re.findall("^(92)", vcode)):
  return "$3.75";
 if(re.findall("^(93)", vcode)):
  return "$4.00";
 if(re.findall("^(94)", vcode)):
  return "Reserved for future use";
 if(re.findall("^(95)", vcode)):
  return "$0.95";
 if(re.findall("^(96)", vcode)):
  return "$4.50";
 if(re.findall("^(97)", vcode)):
  return "Reserved for future use";
 if(re.findall("^(98)", vcode)):
  return "Buy 2 For $0.65";
 if(re.findall("^(99)", vcode)):
  return "$0.99";
 return False;

'''
// Get Major Industry Identifier for Bank Card Number
// Source: https://en.wikipedia.org/wiki/Credit_card_number#Major_Industry_Identifier_.28MII.29
'''
def get_bcn_mii_prefix(upc):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(not re.findall("^(\d{16})", upc)):
  return False;
 if(re.findall("^(0)", upc)):
  return "ISO/TC 68";
 if(re.findall("^(1)", upc)):
  return "Airlines";
 if(re.findall("^(2)", upc)):
  return "Airlines";
 if(re.findall("^(3)", upc)):
  return "Travel and Entertainment and Banking/Financial";
 if(re.findall("^(4)", upc)):
  return "Banking and Financial";
 if(re.findall("^(5)", upc)):
  return "Banking and Financial";
 if(re.findall("^(6)", upc)):
  return "Merchandising and Banking/Financial";
 if(re.findall("^(7)", upc)):
  return "Petroleum";
 if(re.findall("^(8)", upc)):
  return "Healthcare and Telecommunications";
 if(re.findall("^(9)", upc)):
  return "National Assignment";
 return False;

'''
// Get UPS Checkdigit and Info by stebo0728 and HolidayBows
// Source: http://www.codeproject.com/Articles/21224/Calculating-the-UPS-Tracking-Number-Check-Digit
// Source: http://www.codeproject.com/Articles/21224/Calculating-the-UPS-Tracking-Number-Check-Digit?msg=2961884#xx2961884xx
'''
def get_ups_barcode_info(upc, infotype=None):
 upc = str(upc).upper();
 if(not re.findall("^1Z", upc)):
  return False;
 if(re.findall("^1Z", upc)):
  fix_matches = re.findall("^1Z(\w*)", upc);
  upc = fix_matches[0];
 if(len(upc)>16):
  fix_matches = re.findall("^(\w{16})", upc);
  upc = fix_matches[0];
 upc_matches = re.findall("^(\w{6})(\w{2})(\w{5})(\w{2})(\w{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'accountnumber': pre_upc_type[0], 'servicetype': pre_upc_type[1], 'invoicenumber': pre_upc_type[2], 'packagenumber': pre_upc_type[3], 'checkdigit': pre_upc_type[4]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_ups_barcode_accountnumber(upc):
 upc = str(upc).upper();
 product = get_ups_barcode_info(upc);
 if(not product):
  return False;
 return product.get("accountnumber", False);
def get_ups_barcode_servicetype(upc):
 upc = str(upc).upper();
 product = get_ups_barcode_info(upc);
 if(not product):
  return False;
 return product.get("servicetype", False);
def get_ups_barcode_servicetype_info(upc):
 upc = str(upc);
 upc = get_ups_barcode_servicetype(upc);
 if(re.findall("^(01)", upc)):
  return "Next Day Air Shipment";
 if(re.findall("^(02)", upc)):
  return "Second Day Air Shipment";
 if(re.findall("^(03)", upc)):
  return "Ground Shipment";
 return False;
def get_ups_barcode_invoicenumber(upc):
 upc = str(upc).upper();
 product = get_ups_barcode_info(upc);
 if(not product):
  return False;
 return product.get("invoicenumber", False);
def get_ups_barcode_packagenumber(upc):
 upc = str(upc).upper();
 product = get_ups_barcode_info(upc);
 if(not product):
  return False;
 return product.get("packagenumber", False);
def get_ups_barcode_checkdigit(upc):
 upc = str(upc).upper();
 product = get_ups_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);

'''
// Get IMEI (International Mobile Station Equipment Identity) Info
// Source: https://en.wikipedia.org/wiki/Credit_card_number#Major_Industry_Identifier_.28MII.29
'''
def get_new_imei_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(not re.findall("^(\d{16})", upc)):
  return False;
 upc_matches = re.findall("^(\d{8})(\d{6})(\d{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'tac': pre_upc_type[0], 'serialnumber': pre_upc_type[1], 'checkdigit': pre_upc_type[2]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_new_imei_barcode_tac(upc):
 upc = str(upc);
 product = get_new_imei_barcode_info(upc);
 if(not product):
  return False;
 return product.get("tac", False);
def get_new_imei_barcode_serialnumber(upc):
 upc = str(upc);
 product = get_new_imei_barcode_info(upc);
 if(not product):
  return False;
 return product.get("serialnumber", False);
def get_new_imei_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_new_imei_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);
def get_old_imei_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(not re.findall("^(\d{16})", upc)):
  return False;
 upc_matches = re.findall("^(\d{6})(\d{2})(\d{6})(\d{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'tac': pre_upc_type[0], 'fac': pre_upc_type[1], 'serialnumber': pre_upc_type[2], 'checkdigit': pre_upc_type[3]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_old_imei_barcode_tac(upc):
 upc = str(upc);
 product = get_old_imei_barcode_info(upc);
 if(not product):
  return False;
 return product.get("tac", False);
def get_old_imei_barcode_fac(upc):
 upc = str(upc);
 product = get_old_imei_barcode_info(upc);
 if(not product):
  return False;
 return product.get("fac", False);
def get_old_imei_barcode_serialnumber(upc):
 upc = str(upc);
 product = get_old_imei_barcode_info(upc);
 if(not product):
  return False;
 return product.get("serialnumber", False);
def get_old_imei_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_old_imei_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);

'''
// Get IMEISV (International Mobile Station Equipment Identity Software Version) Info
// Source: https://en.wikipedia.org/wiki/Credit_card_number#Major_Industry_Identifier_.28MII.29
'''
def get_new_imeisv_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(not re.findall("^(\d{16})", upc)):
  return False;
 upc_matches = re.findall("^(\d{8})(\d{6})(\d{2})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'tac': pre_upc_type[0], 'serialnumber': pre_upc_type[1], 'svn': pre_upc_type[2]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_new_imeisv_barcode_tac(upc):
 upc = str(upc);
 product = get_new_imeisv_barcode_info(upc);
 if(not product):
  return False;
 return product.get("tac", False);
def get_new_imeisv_barcode_serialnumber(upc):
 upc = str(upc);
 product = get_new_imeisv_barcode_info(upc);
 if(not product):
  return False;
 return product.get("serialnumber", False);
def get_new_imeisv_barcode_svn(upc):
 upc = str(upc);
 product = get_new_imeisv_barcode_info(upc);
 if(not product):
  return False;
 return product.get("svn", False);
def get_old_imeisv_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(not re.findall("^(\d{16})", upc)):
  return False;
 upc_matches = re.findall("^(\d{6})(\d{2})(\d{6})(\d{2})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'tac': pre_upc_type[0], 'fac': pre_upc_type[1], 'serialnumber': pre_upc_type[2], 'svn': pre_upc_type[3]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_old_imeisv_barcode_tac(upc):
 upc = str(upc);
 product = get_old_imeisv_barcode_info(upc);
 if(not product):
  return False;
 return product.get("tac", False);
def get_old_imeisv_barcode_fac(upc):
 upc = str(upc);
 product = get_old_imeisv_barcode_info(upc);
 if(not product):
  return False;
 return product.get("fac", False);
def get_old_imeisv_barcode_serialnumber(upc):
 upc = str(upc);
 product = get_old_imeisv_barcode_info(upc);
 if(not product):
  return False;
 return product.get("serialnumber", False);
def get_old_imeisv_barcode_svn(upc):
 upc = str(upc);
 product = get_old_imeisv_barcode_info(upc);
 if(not product):
  return False;
 return product.get("svn", False);

'''
// Get Bank Card Number Info
// Source: https://en.wikipedia.org/wiki/Credit_card_number#Major_Industry_Identifier_.28MII.29
'''
def get_bcn_info(upc, infotype=None):
 upc = str(upc);
 if(not re.findall("^(\d{16})", upc)):
  return False;
 upc_matches = re.findall("^(\d{1})(\d{5})(\d{12})(\d{1})", upc);
 pre_upc_type = upc_matches[0];
 upc_type = {'mii': pre_upc_type[0], 'iin': pre_upc_type[0]+pre_upc_type[1], 'account': pre_upc_type[2], 'checkdigit': pre_upc_type[3]};
 if(infotype is None):
  return upc_type;
 if(infotype is not None):
  return upc_type.get(infotype, upc_type);
def get_bcn_mii(upc):
 upc = str(upc);
 product = get_bcn_info(upc);
 if(not product):
  return False;
 return product.get("mii", False);
def get_bcn_iin(upc):
 upc = str(upc);
 product = get_bcn_info(upc);
 if(not product):
  return False;
 return product.get("iin", False);
def get_bcn_account(upc):
 upc = str(upc);
 product = get_bcn_info(upc);
 if(not product):
  return False;
 return product.get("account", False);
def get_bcn_checkdigit(upc):
 upc = str(upc);
 product = get_bcn_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);

'''
// Get National Drug Codes UPC-A info
// Source: http://www.drugs.com/ndc.html
// Source: http://www.wikihow.com/Read-12-Digit-UPC-Barcodes
'''
def get_upca_ndc_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(re.findall("^0(\d{12})", upc)):
  upc_matches = re.findall("^0(\d{12})", upc);
  upc = upc_matches[0];
 if(not re.findall("^(\d{12})", upc)):
  return False;
 if(not re.findall("^3(\d{11})", upc)):
  return False;
 upc_matches = re.findall("^3(\d{4})(\d{4})(\d{2})(\d{1})", upc);
 upc_matches = upc_matches[0];
 product = {'numbersystem': str(3), 'labeler': upc_matches[0], 'productcode': upc_matches[1], 'packagecode': upc_matches[2], 'checkdigit': upc_matches[3]};
 if(infotype is None):
  return product;
 if(infotype is not None):
  return product.get(infotype, product);
def get_upca_ndc_barcode_numbersystem(upc):
 upc = str(upc);
 product = get_upca_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("numbersystem", False);
def get_upca_ndc_barcode_labeler(upc):
 upc = str(upc);
 product = get_upca_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("labeler", False);
def get_upca_ndc_barcode_productcode(upc):
 upc = str(upc);
 product = get_upca_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("productcode", False);
def get_upca_ndc_barcode_packagecode(upc):
 upc = str(upc);
 product = get_upca_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("packagecode", False);
def get_upca_ndc_barcode_checkdigit(upc):
 upc = str(upc);
 product = get_upca_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("checkdigit", False);

'''
// Get National Drug Codes info
// Source: http://www.drugs.com/ndc.html
// Source: http://www.wikihow.com/Read-12-Digit-UPC-Barcodes
'''
def get_ndc_barcode_info(upc, infotype=None):
 upc = str(upc);
 if(not re.findall("^(\d{10})", upc)):
  return False;
 if(not re.findall("^(\d{10})", upc)):
  return False;
 upc_matches = re.findall("(\d{4})(\d{4})(\d{2})", upc);
 upc_matches = upc_matches[0];
 product = {'labeler': upc_matches[0], 'productcode': upc_matches[1], 'packagecode': upc_matches[2]};
 if(infotype is None):
  return product;
 if(infotype is not None):
  return product.get(infotype, product);
def get_ndc_barcode_labeler(upc):
 upc = str(upc);
 product = get_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("labeler", False);
def get_ndc_barcode_productcode(upc):
 upc = str(upc);
 product = get_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("productcode", False);
def get_ndc_barcode_packagecode(upc):
 upc = str(upc);
 product = get_ndc_barcode_info(upc);
 if(not product):
  return False;
 return product.get("packagecode", False);



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.
    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
    $FileInfo: shortcuts.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import upcean.decode.barcode, upcean.support;

''' // Shortcut Codes by Kazuki Przyborowski '''
def decode_barcode(bctype,infile,resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 if(bctype not in upcean.support.supported_barcodes("tuple")):
  return False;
 if(hasattr(upcean.decode.barcode, "decode_"+bctype+"_barcode") and callable(getattr(upcean.decode.barcode, "decode_"+bctype+"_barcode"))):
  return getattr(upcean.decode.barcode, "decode_"+bctype+"_barcode")(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(not hasattr(upcean.decode.barcode, "decode_"+bctype+"_barcode") or not callable(getattr(upcean.decode.barcode, "decode_"+bctype+"_barcode"))):
  return False;
 return False;

def validate_decode_barcode(bctype,infile="./barcode.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 if(bctype not in upcean.support.supported_barcodes("tuple")):
  return False;
 if(bctype=="upca" or bctype=="upce" or bctype=="ean13" or bctype=="ean8" or bctype=="itf" or bctype=="itf6" or bctype=="itf14"):
  if(hasattr(upcean.decode.barcode, "validate_decode_"+bctype+"_barcode") and callable(getattr(upcean.decode.barcode, "validate_decode_"+bctype+"_barcode"))):
   return getattr(upcean.decode.barcode, "validate_decode_"+bctype+"_barcode")(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
  if(not hasattr(upcean.decode.barcode, "validate_decode_"+bctype+"_barcode") or not callable(getattr(upcean.decode.barcode, "validate_decode_"+bctype+"_barcode"))):
   return False;
  return False;
 if(bctype!="upca" and bctype!="upce" and bctype!="ean13" and bctype!="ean8" and bctype!="itf" and bctype!="itf6" and bctype!="itf14"):
  return decode_barcode(bctype,infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);

def fix_decode_barcode(bctype,infile="./barcode.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 if(bctype not in upcean.support.supported_barcodes("tuple")):
  return False;
 if(bctype=="upca" or bctype=="upce" or bctype=="ean13" or bctype=="ean8" or bctype=="itf" or bctype=="itf6" or bctype=="itf14"):
  if(hasattr(upcean.decode.barcode, "fix_decode_"+bctype+"_barcode") and callable(getattr(upcean.decode.barcode, "fix_decode_"+bctype+"_barcode"))):
   return getattr(upcean.decode.barcode, "fix_decode_"+bctype+"_barcode")(infile,resize,barheight,barwidth,barcolor,imageoutlib);
  if(not hasattr(upcean.decode.barcode, "fix_decode_"+bctype+"_barcode") or not callable(getattr(upcean.decode.barcode, "fix_decode_"+bctype+"_barcode"))):
   return False;
  return False;
 if(bctype!="upca" and bctype!="upce" and bctype!="ean13" and bctype!="ean8" and bctype!="itf" and bctype!="itf6" and bctype!="itf14"):
  return decode_barcode(bctype,infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: oopfuncs.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import upcean.validate, upcean.convert, upcean.getprefix, upcean.barcodes.getsfname, upcean.support;
pilsupport = upcean.support.check_for_pil();
cairosupport = upcean.support.check_for_cairo();
if(pilsupport or cairosupport):
 import upcean.fonts, upcean.xml, upcean.barcodes;
if(pilsupport):
 import upcean.decode;

''' // Object-oriented classes and functions by Kazuki Przyborowski '''
class barcode:
 ''' // Barcode Types '''
 EAN2="ean2";
 UPCS2="ean2";
 EAN5="ean5";
 UPCS5="ean5";
 UPCA="upca";
 GOODWILL="goodwill";
 UPCE="upce";
 EAN13="ean13"
 EAN8="ean8"
 STF="stf";
 ITF="itf";
 ITF14="itf14";
 CODE11="code11";
 CODE39="code39";
 CODE93="code93";
 CODE128="code128"
 CODABAR="codabar";
 MSI="msi";
 bctype_dict=upcean.support.bctype_dict;
 bctype_dict_alt=upcean.support.bctype_dict_alt;
 bctype_list=upcean.support.bctype_list;
 bctype_tuple=upcean.support.bctype_tuple;
 bctype_name=upcean.support.bctype_name;
 def __init__(self, type=None, code=None):
  if(type is not None):
   self.type = type;
  if(code is not None):
   self.code = code;
  self.filename = "./barcode.png";
  self.size = 1;
  self.hidesn = False;
  self.hidecd = False;
  self.hidetext = False;
  self.textxy = (1, 1, 1);
  self.barheight = (48, 54);
  self.barwidth = 1;
  self.barcolor = (0, 0, 0);
  self.textcolor = (0, 0, 0);
  self.bgcolor = (255, 255, 255);
  self.imageoutlib = "pillow";
  self.return_check = False;
  self.return_type = "dict";
 ''' // support.py funtions '''
 def supported_barcodes(self, return_type=None):
  if(return_type is None):
   return_type = self.return_type;
  return upcean.support.supported_barcodes(return_type);
 def barcode_support(self, return_type=None):
  if(return_type is None):
   return_type = self.return_type;
  return upcean.support.barcode_support(return_type);
 def get_barcode_name(self, barcode_type=None):
  if(barcode_type is None):
   barcode_type = self.type;
  return upcean.support.get_barcode_name(barcode_type);
 def check_for_pil(self):
  return upcean.support.check_for_pil();
 def check_for_pillow(self):
  return upcean.support.check_for_pillow();
 def get_pil_version(self, barcode_type=None):
  if(barcode_type is None):
   barcode_type = self.type;
  return upcean.support.get_pil_version();
 def get_pillow_version(self, barcode_type=None):
  if(barcode_type is None):
   barcode_type = self.type;
  return upcean.support.get_pillow_version();
 def get_python_info(self, barcode_type=None):
  if(barcode_type is None):
   barcode_type = self.type;
  return upcean.support.get_pillow_version();
 ''' // barcodes/__init__.py funtions '''
 if(pilsupport or cairosupport):
  def create_barcode(self, filename=None, size=None):
   if(filename is None):
    filename = self.filename;
   if(size is None):
    size = self.size;
   return upcean.barcodes.create_barcode(self.type, self.code, filename, size, (self.hidesn, self.hidecd, self.hidetext), self.barheight, self.barwidth, self.textxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
  def validate_create_barcode(self, filename=None, size=None):
   if(filename is None):
    filename = self.filename;
   if(size is None):
    size = self.size;
   return upcean.barcodes.validate_create_barcode(self.type, self.code, filename, size, (self.hidesn, self.hidecd, self.hidetext), self.barheight, self.barwidth, self.textxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
  def fix_create_barcode(self, filename=None, size=None):
   if(filename is None):
    filename = self.filename;
   if(size is None):
    size = self.size;
   return upcean.barcodes.fix_create_barcode(self.type, self.code, filename, size, (self.hidesn, self.hidecd, self.hidetext), self.barheight, self.barwidth, self.textxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
  def draw_barcode(self, size=None):
   if(size is None):
    size = self.size;
   return upcean.barcodes.draw_barcode(self.type, self.code, size, (self.hidesn, self.hidecd, self.hidetext), self.barheight, self.barwidth, self.textxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
  def validate_draw_barcode(self, size=None):
   if(size is None):
    size = self.size;
   return upcean.barcodes.validate_draw_barcode(self.type, self.code, size, (self.hidesn, self.hidecd, self.hidetext), self.barheight, self.barwidth, self.textxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
  def fix_draw_barcode(self, size=None):
   if(size is None):
    size = self.size;
   return upcean.barcodes.fix_draw_barcode(self.type, self.code, size, (self.hidesn, self.hidecd, self.hidetext), self.barheight, self.barwidth, self.textxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
 ''' // validate/__init__.py funtions '''
 def validate_checksum(self):
  return upcean.validate.validate_checksum(self.type, self.code, self.return_check);
 def validate_luhn_checksum(self):
  return upcean.validate.validate_luhn_checksum(self.code, self.codelen, self.return_check);
 def validate_usps_checksum(self):
  return upcean.validate.validate_usps_checksum(self.code, self.return_check);
 def validate_ups_checksum(self):
  return upcean.validate.validate_ups_checksum(self.code, self.return_check);
 def validate_fedex_checksum(self):
  return upcean.validate.validate_fedex_checksum(self.code, self.return_check);
 def validate_imei_checksum(self):
  return upcean.validate.validate_imei_checksum(self.code, self.return_check);
 def validate_bcn_checksum(self):
  return upcean.validate.validate_bcn_checksum(self.code, self.return_check);
 def get_checksum(self):
  return upcean.validate.get_checksum(self.type, self.code);
 def get_luhn_checksum(self):
  return upcean.validate.get_luhn_checksum(self.code, self.codelen);
 def get_code11_checksum(self):
  return upcean.validate.get_code11_checksum(self.code);
 def get_code93_checksum(self):
  return upcean.validate.get_code93_checksum(self.code);
 def get_code128_checksum(self):
  return upcean.validate.get_code128_checksum(self.code);
 def get_digital_root(self):
  return upcean.validate.get_digital_root(self.number);
 def get_usps_checksum(self):
  return upcean.validate.get_usps_checksum(self.code);
 def get_ups_checksum(self):
  return upcean.validate.get_ups_checksum(self.code);
 def get_fedex_checksum(self):
  return upcean.validate.get_fedex_checksum(self.code);
 def get_imei_checksum(self):
  return upcean.validate.get_imei_checksum(self.code);
 def get_bcn_checksum(self):
  return upcean.validate.get_bcn_checksum(self.code);
 def fix_checksum(self):
  return upcean.validate.fix_checksum(self.type, self.code);
 def fix_luhn_checksum(self):
  return upcean.validate.fix_luhn_checksum(self.code, self.codelen);
 def fix_usps_checksum(self):
  return upcean.validate.fix_usps_checksum(self.code);
 def fix_ups_checksum(self):
  return upcean.validate.fix_ups_checksum(self.code);
 def fix_fedex_checksum(self):
  return upcean.validate.fix_fedex_checksum(self.code);
 def fix_imei_checksum(self):
  return upcean.validate.fix_imei_checksum(self.code);
 def fix_bcn_checksum(self):
  return upcean.validate.fix_bcn_checksum(self.code);
 ''' // convert/__init__.py funtions '''
 def convert_barcode(self, outtype=None):
  if(outtype is None):
   outtype = self.outtype;
  return upcean.convert.convert_barcode(self.type, outtype, self.code);
 def print_convert_barcode(self, outtype=None):
  if(outtype is None):
   outtype = self.outtype;
  return upcean.convert.print_convert_barcode(self.type, outtype, self.code);
 def make_barcode(self, numbersystem, manufacturer, product):
  if(numbersystem is None):
   numbersystem = self.numbersystem;
  if(manufacturer is None):
   manufacturer = self.manufacturer;
  if(product is None):
   product = self.product;
  return upcean.convert.make_barcode(self.type, self.numbersystem, self.manufacturer, self.product);
 ''' // getsfname.py funtions '''
 if(pilsupport or cairosupport):
  def get_save_filename(self, filename=None):
   if(filename is None):
    filename = self.filename;
   return upcean.barcodes.getsfname.get_save_filename(self.filename);
 ''' // getprefix/__init__.py funtions '''
 def get_barcode_info(self):
  return upcean.getprefix.get_barcode_info(self.type, self.code);

''' // Object-oriented classes and functions by Kazuki Przyborowski '''
class decode:
 ''' // Barcode Types '''
 ''' // Barcode Types '''
 EAN2="ean2";
 UPCS2="ean2";
 EAN5="ean5";
 UPCS5="ean5";
 UPCA="upca";
 GOODWILL="goodwill";
 UPCE="upce";
 EAN13="ean13"
 EAN8="ean8"
 STF="stf";
 ITF="itf";
 ITF14="itf14";
 CODE11="code11";
 CODE39="code39";
 CODE93="code93";
 CODE128="code128"
 CODABAR="codabar";
 MSI="msi";
 bctype_dict=upcean.support.bctype_dict;
 bctype_dict_alt=upcean.support.bctype_dict_alt;
 bctype_list=upcean.support.bctype_list;
 bctype_tuple=upcean.support.bctype_tuple;
 bctype_name=upcean.support.bctype_name;
 def __init__(self, type=None, filename=None):
  if(type is not None):
   self.type = type;
  if(filename is not None):
   self.filename = filename;
  self.code = None;
  self.size = 1;
  self.barheight = (48, 54);
  self.barwidth = 1;
  self.shiftxy = (0, 0);
  self.barcolor = (0, 0, 0);
  self.textcolor = (0, 0, 0);
  self.bgcolor = (255, 255, 255);
  self.imageoutlib = "pillow";
  self.return_check = False;
  self.return_type = "dict";
 if(pilsupport):
  def decode_barcode(self, filename=None, size=None):
   if(filename is None):
    filename = self.filename;
   if(size is None):
    size = self.size;
   self.code = upcean.decode.decode_barcode(self.type, filename, size, self.barheight, self.barwidth, self.shiftxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
   return self.code;
  def validate_decode_barcode(self, filename=None, size=None):
   if(filename is None):
    filename = self.filename;
   if(size is None):
    size = self.size;
   self.code = upcean.decode.decode_barcode(self.type, filename, size, self.barheight, self.barwidth, self.shiftxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
   return self.code;
  def fix_decode_barcode(self, filename=None, size=None):
   if(filename is None):
    filename = self.filename;
   if(size is None):
    size = self.size;
   self.code = upcean.decode.fix_decode_barcode(self.type, filename, size, self.barheight, self.barwidth, self.shiftxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib);
   return self.code;



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: __init__.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import os;

try:
 import pkg_resources;
 pkgres = True;
except ImportError:
 pkgres = False;

if(pkgres):
 fontpathocra = pkg_resources.resource_filename(__name__, "OCRA.otf");
 fontpathocraalt = pkg_resources.resource_filename(__name__, "OCRA.ttf");
 fontpathocrb = pkg_resources.resource_filename(__name__, "OCRB.otf");
 fontpathocrbalt = pkg_resources.resource_filename(__name__, "OCRB.ttf");
 fontpath = os.path.dirname(fontpathocrb);
if(not pkgres):
 fontpathocra = os.path.dirname(__file__)+os.sep+"OCRA.otf";
 fontpathocraalt = os.path.dirname(__file__)+os.sep+"OCRA.ttf";
 fontpathocrb = os.path.dirname(__file__)+os.sep+"OCRB.otf";
 fontpathocrbalt = os.path.dirname(__file__)+os.sep+"OCRB.ttf";
 fontpath = os.path.dirname(fontpathocrb);



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: shortcuts.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import upcean.barcodes.barcode, upcean.xml.files, upcean.support;

''' // Shortcut Codes by Kazuki Przyborowski '''
def create_barcode(bctype,upc,outfile="./barcode.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 if(bctype not in upcean.support.supported_barcodes("tuple")):
  return False;
 if(hasattr(upcean.barcodes.barcode, "create_"+bctype+"_barcode") and callable(getattr(upcean.barcodes.barcode, "create_"+bctype+"_barcode"))):
  return getattr(upcean.barcodes.barcode, "create_"+bctype+"_barcode")(upc,outfile,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
 if(not hasattr(upcean.barcodes.barcode, "create_"+bctype+"_barcode") or not callable(getattr(upcean.barcodes.barcode, "create_"+bctype+"_barcode"))):
  return False;
 return False;
def draw_barcode(bctype,upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 return create_barcode(bctype,upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);

def validate_create_barcode(bctype,upc,outfile="./barcode.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 if(bctype not in upcean.support.supported_barcodes("tuple")):
  return False;
 if(bctype=="upca" or bctype=="goodwill" or bctype=="upce" or bctype=="ean13" or bctype=="ean8" or bctype=="itf" or bctype=="itf6" or bctype=="itf14"):
  if(hasattr(upcean.barcodes.barcode, "validate_create_"+bctype+"_barcode") and callable(getattr(upcean.barcodes.barcode, "validate_create_"+bctype+"_barcode"))):
   return getattr(upcean.barcodes.barcode, "validate_create_"+bctype+"_barcode")(upc,outfile,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
  if(not hasattr(upcean.barcodes.barcode, "validate_create_"+bctype+"_barcode") or not callable(getattr(upcean.barcodes.barcode, "validate_create_"+bctype+"_barcode"))):
   return False;
  return False;
 if(bctype!="upca" and bctype!="goodwill" and bctype!="upce" and bctype!="ean13" and bctype!="ean8" and bctype!="itf" and bctype!="itf6" and bctype!="itf14"):
  return create_barcode(bctype,upc,outfile,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
def validate_draw_barcode(bctype,upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 return validate_create_barcode(bctype,upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);

def fix_create_barcode(bctype,upc,outfile="./barcode.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 if(bctype not in upcean.support.supported_barcodes("tuple")):
  return False;
 if(bctype=="upca" or bctype=="goodwill" or bctype=="upce" or bctype=="ean13" or bctype=="ean8" or bctype=="itf" or bctype=="itf6" or bctype=="itf14"):
  if(hasattr(upcean.barcodes.barcode, "fix_create_"+bctype+"_barcode") and callable(getattr(upcean.barcodes.barcode, "fix_create_"+bctype+"_barcode"))):
   return getattr(upcean.barcodes.barcode, "fix_create_"+bctype+"_barcode")(upc,outfile,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
  if(not hasattr(upcean.barcodes.barcode, "fix_create_"+bctype+"_barcode") or not callable(getattr(upcean.barcodes.barcode, "fix_create_"+bctype+"_barcode"))):
   return False;
  return False;
 if(bctype!="upca" and bctype!="goodwill" and bctype!="upce" and bctype!="ean13" and bctype!="ean8" and bctype!="itf" and bctype!="itf6" and bctype!="itf14"):
  return create_barcode(bctype,upc,outfile,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
def fix_draw_barcode(bctype,upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 return fix_create_barcode(bctype,upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: prepil.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
from PIL import Image, ImageDraw, ImageFont;
import upcean.fonts;

try:
 import pkg_resources;
 pkgres = True;
except ImportError:
 pkgres = False;

fontpathocra = upcean.fonts.fontpathocra;
fontpathocraalt = upcean.fonts.fontpathocraalt;
fontpathocrb = upcean.fonts.fontpathocrb;
fontpathocrbalt = upcean.fonts.fontpathocrbalt;
fontpath = upcean.fonts.fontpath;

''' // Source: http://stevehanov.ca/blog/index.php?id=28 '''
def snapCoords( ctx, x, y ):
 (xd, yd) = ctx.user_to_device(x, y);
 return ( round(x) + 0.5, round(y) + 0.5 );

def drawColorRectangle( ctx, x1, y1, x2, y2, color ):
 ctx.rectangle([(x1, y1), (x2, y2)], fill=color);
 return True;

def drawColorLine( ctx, x1, y1, x2, y2, width, color ):
 if(width < 1): 
  width = 1;
 width -= 1;
 if(width < 1):
  ctx.line((x1, y1, x2, y2), fill=color);
 else:
  ctx.rectangle([(x1, y1), (x2 + width, y2)], fill=color);
 return True;

def drawColorText( ctx, size, x, y, text, color, ftype = "ocrb" ):
 font = ImageFont.truetype(fontpathocra, size);
 if(ftype=="ocra"):
  font = ImageFont.truetype(fontpathocra, size);
 if(ftype=="ocrb"):
  font = ImageFont.truetype(fontpathocrb, size);
 text = str(text);
 ctx.text((x, y), text, font=font, fill=color);
 del(font);
 return True;

def drawColorRectangleAlt( ctx, x1, y1, x2, y2, color ):
 ctx.rectangle([(x1, y1), (x2, y2)], outline=color);
 return True;



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: upce.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import re, upcean.barcodes.getsfname, upcean.support;
from PIL import Image, UnidentifiedImageError;

pilsupport = upcean.support.check_for_pil();
cairosupport = upcean.support.check_for_cairo();
from upcean.barcodes.predraw import *;
if(cairosupport):
 import cairo;

def decode_upce_barcode(infile="./upce.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 if(not re.findall("^([0-9]*[\.]?[0-9])", str(resize)) or int(resize) < 1):
  resize = 1;
 if(isinstance(infile, Image.Image)):
  upc_img = infile.convert('RGB');
 elif(cairosupport and isinstance(infile, cairo.ImageSurface)):
  #upc_img = Image.frombuffer("RGB", (infile.get_width(), infile.get_height()), infile.get_data().tobytes(), "raw", "BGR", 0, 1).convert('RGB');
  #upc_img = Image.frombytes("RGB", (infile.get_width(), infile.get_height()), infile.get_data().tobytes()).convert('RGB');
  return False;
 else:
  try:
   infile.seek(0);
   try:
    upc_img = Image.open(infile).convert('RGB');
   except UnidentifiedImageError:
    return False;
    '''upc_img = Image.frombytes("RGB", (((69 * barwidth) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
  except AttributeError:
   try:
    upc_img = Image.open(infile).convert('RGB');
   except UnidentifiedImageError:
    return False;
    '''prefile = open(infile, "rb");
    upc_img = Image.frombytes("RGB", (((69 * barwidth) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
    prefile.close();'''
 barsize = barwidth * int(resize);
 starty = int(upc_img.size[1] / 2) + shiftxy[1];
 fist_number_dict = { 'EEEOOO': "0", 'EEOEOO': "0", 'EEOOEO': "0", 'EEOOOE': "0", 'EOEEOO': "0", 'EOOEEO': "0", 'EOOOEE': "0", 'EOEOEO': "0", 'EOEOOE': "0", 'EOOEOE': "0", 'OOOEEE': "1", 'OOEOEE': "1", 'OOEEOE': "1", 'OOEEEO': "1", 'OEOOEE': "1", 'OEEOOE': "1", 'OEEEOO': "1", 'OEOEOE': "1", 'OEOEEO': "1", 'OEEOEO': "9" };
 last_number_dict = { 'EEEOOO': "0", 'EEOEOO': "1", 'EEOOEO': "2", 'EEOOOE': "3", 'EOEEOO': "4", 'EOOEEO': "5", 'EOOOEE': "6", 'EOEOEO': "7", 'EOEOOE': "8", 'EOOEOE': "9", 'OOOEEE': "0", 'OOEOEE': "1", 'OOEEOE': "2", 'OOEEEO': "3", 'OEOOEE': "4", 'OEEOOE': "5", 'OEEEOO': "6", 'OEOEOE': "7", 'OEOEEO': "8", 'OEEOEO': "9" };
 left_barcode_o_dict = { '0001101': "0", '0011001': "1", '0010011': "2", '0111101': "3", '0100011': "4", '0110001': "5", '0101111': "6", '0111011': "7", '0110111': "8", '0001011': "9" };
 left_barcode_e_dict = { '0100111': "0", '0110011': "1", '0011011': "2", '0100001': "3", '0011101': "4", '0111001': "5", '0000101': "6", '0010001': "7", '0001001': "8", '0010111': "9" };
 startx = 12;
 if(shiftxy[0] is None):
  prestartx = 0;
  startx = 0;
  gotvalue = False;
  while(prestartx<upc_img.size[0]):
   inprestartx = prestartx;
   substartx = prestartx + (3 * (barwidth * int(resize)));
   curpixelist=[];
   if(upc_img.getpixel((inprestartx, starty))==barcolor[0]):
    if(inprestartx+(2 * (barwidth * int(resize))) > upc_img.size[0]):
     return False;
    curpixelist.append(upc_img.getpixel((inprestartx, starty)));
    curpixelist.append(upc_img.getpixel((inprestartx+(1 * (barwidth * int(resize))), starty)));
    curpixelist.append(upc_img.getpixel((inprestartx+(2 * (barwidth * int(resize))), starty)));
    inprestartx += (3 + 42) * (barwidth * int(resize));
    if(inprestartx+(5 * (barwidth * int(resize))) > upc_img.size[0]):
     return False;
    curpixelist.append(upc_img.getpixel((inprestartx, starty)));
    curpixelist.append(upc_img.getpixel((inprestartx+(1 * (barwidth * int(resize))), starty)));
    curpixelist.append(upc_img.getpixel((inprestartx+(2 * (barwidth * int(resize))), starty)));
    curpixelist.append(upc_img.getpixel((inprestartx+(3 * (barwidth * int(resize))), starty)));
    curpixelist.append(upc_img.getpixel((inprestartx+(4 * (barwidth * int(resize))), starty)));
    curpixelist.append(upc_img.getpixel((inprestartx+(5 * (barwidth * int(resize))), starty)));
    if((curpixelist[0]==barcolor[0] and curpixelist[1]==barcolor[2] and curpixelist[2]==barcolor[0]) and (curpixelist[3]==barcolor[2] and curpixelist[4]==barcolor[0] and curpixelist[5]==barcolor[2] and curpixelist[6]==barcolor[0] and curpixelist[7]==barcolor[2] and curpixelist[8]==barcolor[0])):
     startx = substartx;
     break;
   prestartx += 1;
  shiftxy = (0, shiftxy[1]);
 else:
  startx = ((12 * (barwidth * int(resize))) + shiftxy[0]);
 endx = (3 + 42 + 6) * (barwidth * int(resize));
 listcount = 0;
 pre_upc_whole = [];
 prestartx = startx;
 while(startx < endx):
  listcount = 0;
  pre_upc_list = [];
  while(listcount<7):
   curpixel = upc_img.getpixel((startx, starty));
   if(curpixel==barcolor[0]):
    pre_upc_list.append("1");
   if(curpixel==barcolor[2]):
    pre_upc_list.append("0");
   startx += 1 * (barwidth * int(resize));
   listcount += 1;
  pre_upc_whole.append("".join(pre_upc_list));
 upc_img.close();
 countlist = len(pre_upc_whole);
 listcount = 0;
 barcode_list = [];
 parity_pattern_list = [];
 while(listcount<countlist):
  if(left_barcode_o_dict.get(pre_upc_whole[listcount], None) is not None):
   parity_pattern_list.append("O");
   barcode_list.append(left_barcode_o_dict.get(pre_upc_whole[listcount], "0"));
  elif(left_barcode_e_dict.get(pre_upc_whole[listcount], None) is not None):
   parity_pattern_list.append("E");
   barcode_list.append(left_barcode_e_dict.get(pre_upc_whole[listcount], "0"));
  else:
   return False;
  listcount += 1;
 get_parity_pattern = "".join(parity_pattern_list);
 barcode_list.insert(0, fist_number_dict.get(get_parity_pattern, "0"));
 barcode_list.append(last_number_dict.get(get_parity_pattern, "0"));
 upc = "".join(barcode_list);
 return upc;



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2020 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2020 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: barcode.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import upcean.support, upcean.getprefix.getprefix;

''' // Code for decoding UPC-A by Kazuki Przyborowski '''
from upcean.decode.upca import *;
''' // Code for decoding UPC-E by Kazuki Przyborowski '''
from upcean.decode.upce import *;
''' // Code for decoding EAN-13 by Kazuki Przyborowski '''
from upcean.decode.ean13 import *;
''' // Code for decoding EAN-8 by Kazuki Przyborowski '''
from upcean.decode.ean8 import *;
''' // Code for making Interleaved 2 of 5 by Kazuki Przyborowski '''
from upcean.decode.itf import *;
''' // Code for making ITF-14 by Kazuki Przyborowski '''
from upcean.decode.itf14 import *;

def validate_decode_upca_barcode(infile="./upca.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_upca_barcode(infile,resize,barheight,barwidth,barcolor,imageoutlib);
 if(len(upc)>12 or len(upc)<12):
  return False;
 if(not upcean.validate.validate_upca_checksum(upc)):
  return False;
 return True;

def validate_decode_upce_barcode(infile="./upce.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_upce_barcode(infile,resize,barheight,barwidth,barcolor,imageoutlib);
 if(len(upc)>8 or len(upc)<8):
  return False;
 if(not upcean.validate.validate_upce_checksum(upc)):
  return False;
 return True;

def validate_decode_ean13_barcode(infile="./ean13.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_ean13_barcode(infile,resize,barheight,barwidth,barcolor,imageoutlib);
 if(len(upc)>13 or len(upc)<13):
  return False;
 if(not upcean.validate.validate_ean13_checksum(upc)):
  return False;
 return True;

def validate_decode_itf_barcode(infile="./itf.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_itf_barcode(infile,resize,barheight,barwidth,barcolor,imageoutlib);
 if(len(upc)>14 or len(upc)<14):
  return False;
 if(not upcean.validate.validate_itf14_checksum(upc)):
  return False;
 return True;

def validate_decode_itf6_barcode(infile="./itf6.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_itf6_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>6 or len(upc)<6):
  return False;
 if(not upcean.validate.validate_itf6_checksum(upc)):
  return False;
 return True;

def validate_decode_itf14_barcode(infile="./itf14.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_itf14_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>14 or len(upc)<14):
  return False;
 if(not upcean.validate.validate_itf14_checksum(upc)):
  return False;
 return True;

def fix_decode_upca_barcode(infile="./upca.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_upca_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>12 or len(upc)<11):
  return False;
 upc = upcean.validate.fix_upca_checksum(upc);
 return upc;

def fix_decode_upce_barcode(infile="./upce.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_upce_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>8 or len(upc)<7):
  return False;
 upc = upcean.validate.fix_upce_checksum(upc);
 return upc;

def fix_decode_ean13_barcode(infile="./ean13.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_ean13_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>13 or len(upc)<12):
  return False;
 upc = upcean.validate.fix_ean13_checksum(upc);
 return upc;

def fix_decode_ean8_barcode(infile="./ean8.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_ean8_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>8 or len(upc)<7):
  return False;
 upc = upcean.validate.fix_ean8_checksum(upc);
 return upc;

def fix_decode_itf_barcode(infile="./itf.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_itf14_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>14 or len(upc)<13):
  return False;
 upc = upcean.validate.fix_itf14_checksum(upc);
 return upc;

def fix_decode_itf6_barcode(infile="./itf6.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_itf6_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>6 or len(upc)<5):
  return False;
 upc = upcean.validate.fix_itf6_checksum(upc);
 return upc;

def fix_decode_itf14_barcode(infile="./itf14.png",resize=1,barheight=(48, 54),barwidth=1,shiftxy=(0, 0),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
 upc = decode_itf14_barcode(infile,resize,barheight,barwidth,shiftxy,barcolor,imageoutlib);
 if(len(upc)>14 or len(upc)<13):
  return False;
 upc = upcean.validate.fix_itf14_checksum(upc);
 return upc;



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: validate.py - Last Update: 3/25/2023 Ver. 2.8.8 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals;
import re;

'''
// Digital Root
// Source: http://en.wikipedia.org/wiki/Digital_root
'''
def get_digital_root(number):
 number = str(number);
 while(len(str(number))>1):
  subnum = list(str(number));
  PreCount = 0;
  number = 0;
  while (PreCount<=len(subnum)-1):
   number += int(subnum[PreCount]);
   PreCount += 1;
 return int(number);

'''
// Luhn Algorithm ( Luhn Formula )
// http://en.wikipedia.org/wiki/Luhn_algorithm#Implementation_of_standard_Mod_10
'''
def validate_luhn_checksum(upc, upclen, return_check=False):
 upc = str(upc);
 upclen = int(upclen);
 upclendwn = upclen - 1;
 if(len(upc)>upclen):
  fix_matches = re.findall("^(\d{"+str(upclen)+"})", upc);
  upc = fix_matches[0];
 if(len(upc)>upclen or len(upc)<upclendwn):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_count1 = 0;
 upc_len1 = len(upc_matches1);
 if(len(upc)==upclen and upclen%2==1):
  upc_len1 = len(upc_matches1) - 1;
 OddSum = 0;
 while(upc_count1<upc_len1):
  OddSum = OddSum + upc_matches1[upc_count1];
  upc_count1 = upc_count1 + 1;
 upc_matches2 = upc_matches[1:][::2];
 upc_count2 = 0;
 upc_len2 = len(upc_matches2);
 if(len(upc)==upclen and upclen%2==0):
  upc_len2 = len(upc_matches2) - 1;
 EvenSum = 0;
 while(upc_count2<upc_len2):
  EvenSum = EvenSum + get_digital_root(upc_matches2[upc_count2] * 2);
  upc_count2 = upc_count2 + 1;
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==upclen):
  if(CheckSum!=upc_matches[-1]):
   return False;
  if(CheckSum==upc_matches[-1]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==upclendwn):
  return str(CheckSum);
def get_luhn_checksum(upc, upclen):
 upc = str(upc);
 upclen = int(upclen);
 upclendwn = upclen - 1;
 return validate_luhn_checksum(upc,upclen,True);
def fix_luhn_checksum(upc, upclen):
 upc = str(upc);
 upclen = int(upclen);
 upclendwn = upclen - 1;
 if(len(upc)>upclendwn):
  fix_matches = re.findall("^(\d{"+str(upclendwn)+"})", upc); 
  upc = fix_matches[0];
 return upc+str(get_luhn_checksum(upc,upclen));

def validate_upca_checksum(upc, return_check=False): 
 upc = str(upc);
 if(len(upc)>12):
  fix_matches = re.findall("^(\d{12})", upc);
  upc = fix_matches[0];
 if(len(upc)>12 or len(upc)<11):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 OddSum = (upc_matches1[0] + upc_matches1[1] + upc_matches1[2] + upc_matches1[3] + upc_matches1[4] + upc_matches1[5]) * 3;
 EvenSum = upc_matches2[0] + upc_matches2[1] + upc_matches2[2] + upc_matches2[3] + upc_matches2[4];
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==12):
  if(CheckSum!=upc_matches2[5]):
   return False;
  if(CheckSum==upc_matches2[5]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==11):
  return str(CheckSum);
def get_upca_checksum(upc):
 upc = str(upc);
 return validate_upca_checksum(upc,True);
def fix_upca_checksum(upc):
 upc = str(upc);
 if(len(upc)>11):
  fix_matches = re.findall("^(\d{11})", upc); 
  upc = fix_matches[0];
 return upc+str(get_upca_checksum(upc));

def validate_ean13_checksum(upc, return_check=False):
 upc = str(upc);
 if(len(upc)>13):
  fix_matches = re.findall("^(\d{13})", upc);
  upc = fix_matches[0];
 if(len(upc)>13 or len(upc)<12):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 EvenSum = (upc_matches2[0] + upc_matches2[1] + upc_matches2[2] + upc_matches2[3] + upc_matches2[4] + upc_matches2[5]) * 3;
 OddSum = upc_matches1[0] + upc_matches1[1] + upc_matches1[2] + upc_matches1[3] + upc_matches1[4] + upc_matches1[5];
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==13):
  if(CheckSum!=upc_matches1[6]):
   return False;
  if(CheckSum==upc_matches1[6]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==12):
  return str(CheckSum);
def get_ean13_checksum(upc):
 upc = str(upc);
 return validate_ean13_checksum(upc,True);
def fix_ean13_checksum(upc):
 upc = str(upc);
 if(len(upc)>12):
  fix_matches = re.findall("^(\d{12})", upc); 
  upc = fix_matches[0];
 return upc+str(get_ean13_checksum(upc));

def validate_itf6_checksum(upc, return_check=False):
 upc = str(upc);
 if(len(upc)>6):
  fix_matches = re.findall("^(\d{6})", upc); 
  upc = fix_matches[0];
 if(len(upc)>6 or len(upc)<5):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 CheckSum = 10 - ( ( 3 * upc_matches[0] + upc_matches[1] + 3 * upc_matches[2] + upc_matches[3] + 3 * upc_matches[4] ) % 10 );
 if(not return_check and len(upc)==6):
  if(CheckSum!=upc_matches[5]):
   return False;
  if(CheckSum==upc_matches[5]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==5):
  return str(CheckSum);
def get_itf6_checksum(upc):
 upc = str(upc);
 return validate_itf6_checksum(upc,True);
def fix_itf6_checksum(upc):
 upc = str(upc);
 if(len(upc)>5):
  fix_matches = re.findall("^(\d{5})", upc); 
  upc = fix_matches[0];
 return upc+str(get_itf6_checksum(upc));

def validate_itf14_checksum(upc, return_check=False):
 upc = str(upc);
 if(len(upc)>14):
  fix_matches = re.findall("^(\d{14})", upc); 
  upc = fix_matches[0];
 if(len(upc)>14 or len(upc)<13):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 EvenSum = upc_matches2[0] + upc_matches2[1] + upc_matches2[2] + upc_matches2[3] + upc_matches2[4] + upc_matches2[5];
 OddSum = (upc_matches1[0] + upc_matches1[1] + upc_matches1[2] + upc_matches1[3] + upc_matches1[4] + upc_matches1[5] + upc_matches1[6]) * 3;
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==14):
  if(CheckSum!=upc_matches2[6]):
   return False;
  if(CheckSum==upc_matches2[6]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==13):
  return str(CheckSum);
def get_itf14_checksum(upc):
 upc = str(upc);
 return validate_itf14_checksum(upc,True);
def fix_itf14_checksum(upc):
 upc = str(upc);
 if(len(upc)>13):
  fix_matches = re.findall("^(\d{13})", upc); 
  upc = fix_matches[0];
 return upc+str(get_itf14_checksum(upc));

def validate_ean8_checksum(upc, return_check=False):
 upc = str(upc);
 if(len(upc)>8):
  fix_matches = re.findall("^(\d{8})", upc); 
  upc = fix_matches[0];
 if(len(upc)>8 or len(upc)<7):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 EvenSum = (upc_matches1[0] + upc_matches1[1] + upc_matches1[2] + upc_matches1[3]) * 3;
 OddSum = upc_matches2[0] + upc_matches2[1] + upc_matches2[2];
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==8):
  if(CheckSum!=upc_matches2[3]):
   return False;
  if(CheckSum==upc_matches2[3]): 
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==7):
  return str(CheckSum);
def get_ean8_checksum(upc):
 upc = str(upc);
 return validate_ean8_checksum(upc,True);
def fix_ean8_checksum(upc):
 upc = str(upc);
 if(len(upc)>7):
  fix_matches = re.findall("^(\d{7})", upc); 
  upc = fix_matches[0];
 return upc+str(get_ean8_checksum(upc));

def validate_upce_checksum(upc, return_check=False):
 upc = str(upc);
 if(len(upc)>8):
  fix_matches = re.findall("/^(\d{8})/", upc); 
  upc = fix_matches[0];
 if(len(upc)>8 or len(upc)<7):
  return False;
 if(not re.findall("^(0|1)", upc)):
  return False;
 CheckDigit = None;
 if(len(upc)==8 and re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc)):
  upc_matches = re.findall("^(\d{7})(\d{1})", upc);
  upc_matches = upc_matches[0];
  upc_matches = [int(x) for x in upc_matches];
  CheckDigit = upc_matches[1];
 if(re.findall("^(\d{1})(\d{5})([0-3])", upc)):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc);
  upc_matches = upc_matches[0];
  upc_matches = [int(x) for x in upc_matches];
  if(int(upc_matches[6])==0):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[3] + upc_matches[5]) * 3;
   EvenSum = upc_matches[1] + upc_matches[4];
  if(int(upc_matches[6])==1):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[3] + upc_matches[5]) * 3;
   EvenSum = upc_matches[1] + 1 + upc_matches[4];
  if(int(upc_matches[6])==2):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[3] + upc_matches[5]) * 3;
   EvenSum = upc_matches[1] + 2 + upc_matches[4];
  if(int(upc_matches[6])==3):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[5]) * 3;
   EvenSum = upc_matches[1] + upc_matches[3] + upc_matches[4];
 if(re.findall("^(\d{1})(\d{5})([4-9])", upc)):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc);
  upc_matches = upc_matches[0];
  upc_matches = [int(x) for x in upc_matches];
  if(int(upc_matches[6])==4):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[4] + upc_matches[5]) * 3;
   EvenSum = upc_matches[1] + upc_matches[3];
  if(int(upc_matches[6])==5):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[4] + upc_matches[6]) * 3;
   EvenSum = upc_matches[1] + upc_matches[3] + upc_matches[5];
  if(int(upc_matches[6])==6):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[4] + upc_matches[6]) * 3;
   EvenSum = upc_matches[1] + upc_matches[3] + upc_matches[5];
  if(int(upc_matches[6])==7):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[4] + upc_matches[6]) * 3;
   EvenSum = upc_matches[1] + upc_matches[3] + upc_matches[5];
  if(int(upc_matches[6])==8):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[4] + upc_matches[6]) * 3;
   EvenSum = upc_matches[1] + upc_matches[3] + upc_matches[5];
  if(int(upc_matches[6])==9):
   OddSum = (upc_matches[0] + upc_matches[2] + upc_matches[4] + upc_matches[6]) * 3;
   EvenSum = upc_matches[1] + upc_matches[3] + upc_matches[5];
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==8):
  if(CheckSum!=CheckDigit):
   return False;
  if(CheckSum==CheckDigit):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==7):
  return str(CheckSum);
def get_upce_checksum(upc):
 upc = str(upc);
 return validate_upce_checksum(upc,True);
def fix_upce_checksum(upc):
 upc = str(upc);
 if(len(upc)>7):
  fix_matches = re.findall("^(\d{7})", upc); 
  upc = fix_matches[0];
 return upc+str(get_upce_checksum(upc));

def validate_ean2_checksum(upc, return_check=False): 
 upc = str(upc);
 if(len(upc)>3):
  fix_matches = re.findall("^(\d{3})", upc);
  upc = fix_matches[0];
 if(len(upc)>3 or len(upc)<2):
  return False;
 if(len(upc)==2):
  upc_matches = re.findall("^(\d{2})", upc);
  upc_matches = [int(x) for x in upc_matches];
 if(len(upc)==3):
  upc_matches = re.findall("^(\d{2})(\d{1})", upc);
  upc_matches = upc_matches[0];
  upc_matches = [int(x) for x in upc_matches];
 if(len(upc_matches)<=0): 
  return False;
 CheckSum = upc_matches[0] % 4;
 if(not return_check and len(upc)==3):
  if(CheckSum!=upc_matches[1]):
   return False;
  if(CheckSum==upc_matches[1]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==2):
  return str(CheckSum);
def get_ean2_checksum(upc):
 upc = str(upc);
 return validate_ean2_checksum(upc,True);
def fix_ean2_checksum(upc):
 upc = str(upc);
 if(len(upc)>2):
  fix_matches = re.findall("^(\d{2})", upc); 
  upc = fix_matches[0];
 return upc+str(get_ean2_checksum(upc));

def validate_ean5_checksum(upc, return_check=False): 
 upc = str(upc);
 if(len(upc)>6):
  fix_matches = re.findall("^(\d{6})", upc);
  upc = fix_matches[0];
 if(len(upc)>6 or len(upc)<5):
  return False;
 if(len(upc)==5):
  upc_matches = re.findall("^(\d{5})", upc);
 if(len(upc)==6):
  upc_matches = re.findall("^(\d{5})(\d{1})", upc);
  upc_matches = upc_matches[0];
 if(len(upc_matches)<=0): 
  return False;
 LeftDigit = list(upc_matches[0]);
 LeftDigit = [int(x) for x in LeftDigit];
 CheckSum = (LeftDigit[0] * 3) + (LeftDigit[1] * 9) + (LeftDigit[2] * 3) + (LeftDigit[3] * 9) + (LeftDigit[4] * 3);
 CheckSum = CheckSum % 10;
 upc_matches = [int(x) for x in upc_matches];
 if(not return_check and len(upc)==6):
  if(CheckSum!=upc_matches[1]):
   return False;
  if(CheckSum==upc_matches[1]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==5):
  return str(CheckSum);
def get_ean5_checksum(upc):
 upc = str(upc);
 return validate_ean5_checksum(upc,True);
def fix_ean5_checksum(upc):
 upc = str(upc);
 if(len(upc)>5):
  fix_matches = re.findall("^(\d{5})", upc); 
  upc = fix_matches[0];
 return upc+str(get_ean5_checksum(upc));

'''
// Get USPS Checkdigit by MACY8167
// Source: http://www.mrexcel.com/forum/excel-questions/530675-usps-mod-10-check-digit.html
'''
def validate_usps_checksum(upc, return_check=False): 
 upc = str(upc);
 if(len(upc)>22):
  fix_matches = re.findall("^(\d{22})", upc);
  upc = fix_matches[0];
 if(len(upc)>22 or len(upc)<21):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 OddSum = (upc_matches1[0] + upc_matches1[1] + upc_matches1[2] + upc_matches1[3] + upc_matches1[4] + upc_matches1[5] + upc_matches1[6] + upc_matches1[7] + upc_matches1[8] + upc_matches1[9] + upc_matches1[10]) * 3;
 EvenSum = upc_matches2[0] + upc_matches2[1] + upc_matches2[2] + upc_matches2[3] + upc_matches2[4] + upc_matches2[5] + upc_matches2[6] + upc_matches2[7] + upc_matches2[8] + upc_matches2[9];
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==22):
  if(CheckSum!=upc_matches2[10]):
   return False;
  if(CheckSum==upc_matches2[10]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==21):
  return str(CheckSum);
def get_usps_checksum(upc):
 upc = str(upc);
 return validate_usps_checksum(upc,True);
def fix_usps_checksum(upc):
 upc = str(upc);
 if(len(upc)>21):
  fix_matches = re.findall("^(\d{21})", upc); 
  upc = fix_matches[0];
 return upc+str(get_usps_checksum(upc));

'''
// Get UPS Checkdigit and Info by stebo0728 and HolidayBows
// Source: http://www.codeproject.com/Articles/21224/Calculating-the-UPS-Tracking-Number-Check-Digit
// Source: http://www.codeproject.com/Articles/21224/Calculating-the-UPS-Tracking-Number-Check-Digit?msg=2961884#xx2961884xx
'''
def validate_ups_checksum(upc, return_check=False): 
 upc = str(upc).upper();
 if(not re.findall("^1Z", upc)):
  return False;
 if(re.findall("^1Z", upc)):
  fix_matches = re.findall("^1Z(\w*)", upc);
  upc = fix_matches[0];
 if(len(upc)>16):
  fix_matches = re.findall("^(\w{16})", upc);
  upc = fix_matches[0];
 if(len(upc)>16 or len(upc)<15):
  return False;
 if(len(upc)>16):
  fix_matches = re.findall("^(\w{16})", upc);
  upc = fix_matches[0];
 if(len(upc)>16 or len(upc)<15):
  return False;
 upc_matches = list(upc);
 upc_matches1 = upc_matches[0:][::2];
 upc_count1 = 0;
 OddSum = 0;
 while(upc_count1<8):
  if(upc_matches1[upc_count1].isdigit()):
   OddSum = OddSum + int(upc_matches1[upc_count1]);
  if(not upc_matches1[upc_count1].isdigit()):
   OddSum = OddSum + ((ord(upc_matches1[upc_count1]) - 63) % 10);
  upc_count1 = upc_count1 + 1;
 upc_matches2 = upc_matches[1:][::2];
 upc_count2 = 0;
 EvenSum = 0;
 while(upc_count2<7):
  if(upc_matches2[upc_count2].isdigit()):
   EvenSum = EvenSum + (int(upc_matches2[upc_count2]) * 2);
  if(not upc_matches2[upc_count2].isdigit()):
   EvenSum = EvenSum + (((ord(upc_matches2[upc_count2]) - 63) % 10) * 2);
  upc_count2 = upc_count2 + 1;
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 10;
 if(CheckSum>0):
  CheckSum = 10 - CheckSum;
 if(not return_check and len(upc)==16):
  if(CheckSum!=int(upc_matches2[7])):
   return False;
  if(CheckSum==int(upc_matches2[7])):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==15):
  return str(CheckSum);
def get_ups_checksum(upc):
 upc = str(upc);
 print(upc);
 return validate_ups_checksum(upc,True);
def fix_ups_checksum(upc):
 upc = str(upc);
 if(len(upc)>17):
  fix_matches = re.findall("^(\w{17})", upc); 
  upc = fix_matches[0];
 return upc+str(get_ups_checksum(upc));

'''
// Get FEDEX Checkdigit by jbf777-ga
// Source: http://answers.google.com/answers/threadview/id/207899.html
'''
def validate_fedex_checksum(upc, return_check=False): 
 upc = str(upc);
 if(len(upc)>12):
  fix_matches = re.findall("^(\d{12})", upc);
  upc = fix_matches[0];
 if(len(upc)>12 or len(upc)<11):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 OddSum = (upc_matches1[0] * 3) + (upc_matches1[1] * 7) + (upc_matches1[2] * 1) + (upc_matches1[3] * 3) + (upc_matches1[4] * 7) + (upc_matches1[5] * 1);
 EvenSum = (upc_matches2[0] * 1) + (upc_matches2[1] * 3) + (upc_matches2[2] * 7) + (upc_matches2[3] * 1) + (upc_matches2[4] * 3);
 AllSum = OddSum + EvenSum;
 CheckSum = AllSum % 11;
 if(not return_check and len(upc)==12):
  if(CheckSum!=upc_matches2[5]):
   return False;
  if(CheckSum==upc_matches2[5]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==11):
  return str(CheckSum);
def get_fedex_checksum(upc):
 upc = str(upc);
 return validate_fedex_checksum(upc,True);
def fix_fedex_checksum(upc):
 upc = str(upc);
 if(len(upc)>11):
  fix_matches = re.findall("^(\d{11})", upc); 
  upc = fix_matches[0];
 return upc+str(get_fedex_checksum(upc));

'''
// IMEI (International Mobile Station Equipment Identity)
// Source: http://en.wikipedia.org/wiki/IMEI#Check_digit_computation
'''
def validate_imei_checksum(upc, return_check=False):
 upc = str(upc);
 if(len(upc)>15):
  fix_matches = re.findall("^(\d{15})", upc);
  upc = fix_matches[0];
 if(len(upc)>15 or len(upc)<14):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 UPC_Sum = upc_matches1[0] + get_digital_root(upc_matches2[0] * 2) + upc_matches1[1] + get_digital_root(upc_matches2[1] * 2) + upc_matches1[2] + get_digital_root(upc_matches2[2] * 2) + upc_matches1[3] + get_digital_root(upc_matches2[3] * 2) + upc_matches1[4] + get_digital_root(upc_matches2[4] * 2) + upc_matches1[5] + get_digital_root(upc_matches2[5] * 2) + upc_matches1[6] + get_digital_root(upc_matches2[6] * 2);
 PreCheckSum = 0;
 while((UPC_Sum + PreCheckSum) % 10 != 0):
  PreCheckSum += 1;
 CheckSum = PreCheckSum;
 if(not return_check and len(upc)==15):
  if(CheckSum!=upc_matches1[7]):
   return False;
  if(CheckSum==upc_matches1[7]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==14):
  return str(CheckSum);
def get_imei_checksum(upc):
 upc = str(upc);
 return validate_imei_checksum(upc,True);
def fix_imei_checksum(upc):
 upc = str(upc);
 if(len(upc)>14):
  fix_matches = re.findall("^(\d{14})", upc); 
  upc = fix_matches[0];
 return upc+str(get_imei_checksum(upc));

'''
// Bank Card Numbers
// Source: http://tywkiwdbi.blogspot.com/2012/06/checksum-number-on-credit-card.html
// Source: http://en.wikipedia.org/wiki/Luhn_algorithm#Implementation_of_standard_Mod_10
'''
def validate_bcn_checksum(upc, return_check=False):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>16):
  fix_matches = re.findall("^(\d{16})", upc);
  upc = fix_matches[0];
 if(len(upc)>16 or len(upc)<15):
  return False;
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 upc_matches1 = upc_matches[0:][::2];
 upc_matches2 = upc_matches[1:][::2];
 UPC_Sum = upc_matches2[0] + get_digital_root(upc_matches1[0] * 2) + upc_matches2[1] + get_digital_root(upc_matches1[1] * 2) + upc_matches2[2] + get_digital_root(upc_matches1[2] * 2) + upc_matches2[3] + get_digital_root(upc_matches1[3] * 2) + upc_matches2[4] + get_digital_root(upc_matches1[4] * 2) + upc_matches2[5] + get_digital_root(upc_matches1[5] * 2) + upc_matches2[6] + get_digital_root(upc_matches1[6] * 2) + get_digital_root(upc_matches1[7] * 2);
 PreCheckSum = 0;
 while((UPC_Sum + PreCheckSum) % 10 != 0):
  PreCheckSum += 1;
 CheckSum = PreCheckSum;
 if(not return_check and len(upc)==16):
  if(CheckSum!=upc_matches2[7]):
   return False;
  if(CheckSum==upc_matches2[7]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==15):
  return str(CheckSum);
def get_bcn_checksum(upc):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 return validate_bcn_checksum(upc,True);
def fix_bcn_checksum(upc):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>15):
  fix_matches = re.findall("^(\d{15})", upc); 
  upc = fix_matches[0];
 return upc+str(get_bcn_checksum(upc));

'''
// Code 11
// Source: http://www.barcodeisland.com/code11.phtml
// Source: http://en.wikipedia.org/wiki/Code_11
'''
def get_code11_checksum(upc):
 if(len(upc) < 1): 
  return False;
 if(not re.findall("([0-9\-]+)", upc)):
  return False;
 upc = upc.upper();
 upc_matches = list(upc);
 if(len(upc_matches)<=0):
  return False;
 Code11Array = {0: "0", 1: "1", 2: "2", 3: "3", 4: "4", 5: "5", 6: "6", 7: "7", 8: "8", 9: "9", 10: "-"};
 Code11Values = dict(zip(Code11Array.values(),Code11Array));
 upc_reverse = list(upc_matches);
 upc_reverse.reverse();
 upc_print = list(upc_matches);
 UPC_Count = 0; 
 UPC_Weight = 1; 
 UPC_Sum = 0;
 while (UPC_Count < len(upc_reverse)):
  if(UPC_Weight>10):
   UPC_Weight = 1;
  UPC_Sum += (UPC_Weight * Code11Values[str(upc_reverse[UPC_Count])]);
  UPC_Count += 1; 
  UPC_Weight += 1;
 CheckSum = str(Code11Array[UPC_Sum % 11]);
 upc_reverse = list(upc_matches);
 upc_reverse.reverse();
 UPC_Count = 0; 
 UPC_Weight = 1; 
 UPC_Sum = 0;
 while (UPC_Count < len(upc_reverse)):
  if(UPC_Weight>9):
   UPC_Weight = 1;
  UPC_Sum += (UPC_Weight * Code11Values[str(upc_reverse[UPC_Count])]);
  UPC_Count += 1; 
  UPC_Weight += 1;
 CheckSum = str(CheckSum)+str(Code11Array[UPC_Sum % 11]);
 return str(CheckSum);

'''
// Code 39
// Source: http://www.barcodeisland.com/code39.phtml
// Source: http://en.wikipedia.org/wiki/Code_39
'''
def get_code39_checksum_mod10(upc):
 if(len(upc) < 1): 
  return False;
 if(not re.findall("([0-9a-zA-Z\-\.\$\/\+% ]+)", upc)):
  return False;
 upc = upc.upper();
 upc_matches = list(upc);
 if(len(upc_matches)<=0):
  return False;
 Code39Array = {0: "0", 1: "1", 2: "2", 3: "3", 4: "4", 5: "5", 6: "6", 7: "7", 8: "8", 9: "9", 10: "A", 11: "B", 12: "C", 13: "D", 14: "E", 15: "F", 16: "G", 17: "H", 18: "I", 19: "J", 20: "K", 21: "L", 22: "M", 23: "N", 24: "O", 25: "P", 26: "Q", 27: "R", 28: "S", 29: "T", 30: "U", 31: "V", 32: "W", 33: "X", 34: "Y", 35: "Z", 36: "-", 37: ".", 38: " ", 39: "$", 40: "/", 41: "+", 42: "%"};
 Code39Values = dict(zip(Code39Array.values(),Code39Array));
 upc_print = list(upc_matches);
 UPC_Count = 0; 
 UPC_Sum = 0;
 while (UPC_Count < len(upc_matches)):
  UPC_Sum += Code39Values[str(upc_matches[UPC_Count])];
  UPC_Count += 1; 
 CheckSum = str(Code39Array[UPC_Sum % 10]);
 return str(CheckSum);

def get_code39_checksum_mod43(upc):
 if(len(upc) < 1): 
  return False;
 if(not re.findall("([0-9a-zA-Z\-\.\$\/\+% ]+)", upc)):
  return False;
 upc = upc.upper();
 upc_matches = list(upc);
 if(len(upc_matches)<=0):
  return False;
 Code39Array = {0: "0", 1: "1", 2: "2", 3: "3", 4: "4", 5: "5", 6: "6", 7: "7", 8: "8", 9: "9", 10: "A", 11: "B", 12: "C", 13: "D", 14: "E", 15: "F", 16: "G", 17: "H", 18: "I", 19: "J", 20: "K", 21: "L", 22: "M", 23: "N", 24: "O", 25: "P", 26: "Q", 27: "R", 28: "S", 29: "T", 30: "U", 31: "V", 32: "W", 33: "X", 34: "Y", 35: "Z", 36: "-", 37: ".", 38: " ", 39: "$", 40: "/", 41: "+", 42: "%"};
 Code39Values = dict(zip(Code39Array.values(),Code39Array));
 upc_print = list(upc_matches);
 UPC_Count = 0; 
 UPC_Sum = 0;
 while (UPC_Count < len(upc_matches)):
  UPC_Sum += Code39Values[str(upc_matches[UPC_Count])];
  UPC_Count += 1; 
 CheckSum = str(Code39Array[UPC_Sum % 43]);
 return str(CheckSum);

def get_code39_checksum(upc, getmod="43"):
 getmod = str(getmod);
 if(getmod!="10" and getmod!="43"):
  getmod = "43";
 if(getmod=="10"):
  return get_code39_checksum_mod10(upc);
 if(getmod=="43"):
  return get_code39_checksum_mod43(upc);
 return False;

'''
// Code 93
// Source: http://www.barcodeisland.com/code93.phtml
// Source: http://en.wikipedia.org/wiki/Code_93
'''
def get_code93_checksum(upc):
 if(len(upc) < 1): 
  return False;
 if(not re.findall("([0-9a-zA-Z\-\.\$\/\+% ]+)", upc)):
  return False;
 upc = upc.upper();
 upc_matches = list(upc);
 if(len(upc_matches)<=0):
  return False;
 Code93Array = {0: "0", 1: "1", 2: "2", 3: "3", 4: "4", 5: "5", 6: "6", 7: "7", 8: "8", 9: "9", 10: "A", 11: "B", 12: "C", 13: "D", 14: "E", 15: "F", 16: "G", 17: "H", 18: "I", 19: "J", 20: "K", 21: "L", 22: "M", 23: "N", 24: "O", 25: "P", 26: "Q", 27: "R", 28: "S", 29: "T", 30: "U", 31: "V", 32: "W", 33: "X", 34: "Y", 35: "Z", 36: "-", 37: ".", 38: " ", 39: "$", 40: "/", 41: "+", 42: "%", 43: "($)", 44: "(%)", 45: "(/)", 46: "(+)"};
 Code93Values = dict(zip(Code93Array.values(),Code93Array));
 upc_reverse = list(upc_matches);
 upc_reverse.reverse();
 upc_print = list(upc_matches);
 UPC_Count = 0; 
 UPC_Weight = 1; 
 UPC_Sum = 0;
 while (UPC_Count < len(upc_reverse)):
  if(UPC_Weight>20):
   UPC_Weight = 1;
  UPC_Sum += (UPC_Weight * Code93Values[str(upc_reverse[UPC_Count])]);
  UPC_Count += 1; 
  UPC_Weight += 1;
 CheckSum = str(Code93Array[UPC_Sum % 47]);
 upc_reverse = list(upc_matches);
 upc_reverse.reverse();
 UPC_Count = 0; 
 UPC_Weight = 1; 
 UPC_Sum = 0;
 while (UPC_Count < len(upc_reverse)):
  if(UPC_Weight>15):
   UPC_Weight = 1;
  UPC_Sum += (UPC_Weight * Code93Values[str(upc_reverse[UPC_Count])]);
  UPC_Count += 1; 
  UPC_Weight += 1;
 CheckSum = str(CheckSum)+str(Code93Array[UPC_Sum % 47]);
 return str(CheckSum);

'''
// Code 93
// Source: http://www.barcodeisland.com/code128.phtml
// Source: http://en.wikipedia.org/wiki/Code_128
'''
def get_code128_checksum(upc):
 if(len(upc) % 2):
  return False;
 if(len(upc) < 6): 
  return False;
 if(not re.findall("([0-9a-f]+)", upc)):
  return False;
 upc = upc.lower();
 if(not re.findall("[0-9a-f]{2}", upc)): 
  return False;
 upc_matches = re.findall("[0-9a-f]{2}", upc);
 upc_to_dec = list([int(x, 16) for x in upc_matches]);
 icount = 1;
 icountadd = 1;
 checksum = 0;
 if(upc_to_dec[0]>102 and upc_to_dec[0]<106):
  checksum = checksum + upc_to_dec[0];
  icount = 1;
 else:
  checksum = 0;
  icount = 0;
 upc_less_count = len(upc_to_dec) - 1;
 while(icount < len(upc_to_dec)):
  if(icount==upc_less_count and (upc_to_dec[icount]>105 and upc_to_dec[icount]<108)):
   checksum = checksum;
  else:
   checksum = checksum + (upc_to_dec[icount] * icountadd);
  icount = icount + 1;
  icountadd = icountadd + 1;
 checksum = str(format(checksum % 103, 'x')).zfill(2);
 return checksum;

def convert_hex_code128_to_ascii_code128(upc):
 upc = str(upc);
 if(len(upc) < 8): 
  return False;
 hextoascii = { '00': " ", '01': "!", '02': "\"", '03': "#", '04': "$", '05': "%", '06': "&", '07': "'", '08': "(", '09': ")", '0a': "*", '0b': "+", '0c': ",", '0d': "-", '0e': ".", '0f': "/", '10': "0", '11': "1", '12': "2", '13': "3", '14': "4", '15': "5", '16': "6", '17': "7", '18': "8", '19': "9", '1a': ":", '1b': ";", '1c': "<", '1d': "=", '1e': ">", '1f': "?", '20': "@", '21': "A", '22': "B", '23': "C", '24': "D", '25': "E", '26': "F", '27': "G", '28': "H", '29': "I", '2a': "J", '2b': "K", '2c': "L", '2d': "M", '2e': "N", '2f': "O", '30': "P", '31': "Q", '32': "R", '33': "S", '34': "T", '35': "U", '36': "V", '37': "W", '38': "X", '39': "Y", '3a': "Z", '3b': "[", '3c': "\\", '3d': "]", '3e': "^", '3f': "_", '40': "`", '41': "a", '42': "b", '43': "c", '44': "d", '45': "e", '46': "f", '47': "g", '48': "h", '49': "i", '4a': "j", '4b': "k", '4c': "l", '4d': "m", '4e': "n", '4f': "o", '50': "p", '51': "q", '52': "r", '53': "s", '54': "t", '55': "u", '56': "v", '57': "w", '58': "x", '59': "y", '5a': "z", '5b': "{", '5c': "|", '5d': "}", '5e': "~", '5f': "Ã", '60': "Ä", '61': "Å", '62': "Æ", '63': "Ç", '64': "È", '65': "É", '66': "Ê", '67': "Ë", '68': "Ì", '69': "Í", '6a': "Î", '6b': "Ï", '6c': "Î" };
 asciitohex = { ' ': "00", '!': "01", '"': "02", '#': "03", '$': "04", '%': "05", '&': "06", '\'': "07", '(': "08", ')': "09", '*': "0a", '+': "0b", ',': "0c", '-': "0d", '.': "0e", '/': "0f", '0': "10", '1': "11", '2': "12", '3': "13", '4': "14", '5': "15", '6': "16", '7': "17", '8': "18", '9': "19", ':': "1a", ';': "1b", '<': "1c", '=': "1d", '>': "1e", '?': "1f", '@': "20", 'A': "21", 'B': "22", 'C': "23", 'D': "24", 'E': "25", 'F': "26", 'G': "27", 'H': "28", 'I': "29", 'J': "2a", 'K': "2b", 'L': "2c", 'M': "2d", 'N': "2e", 'O': "2f", 'P': "30", 'Q': "31", 'R': "32", 'S': "33", 'T': "34", 'U': "35", 'V': "36", 'W': "37", 'X': "38", 'Y': "39", 'Z': "3a", '[': "3b", '\\': "3c", ']': "3d", '^': "3e", '_': "3f", '`': "40", 'a': "41", 'b': "42", 'c': "43", 'd': "44", 'e': "45", 'f': "46", 'g': "47", 'h': "48", 'i': "49", 'j': "4a", 'k': "4b", 'l': "4c", 'm': "4d", 'n': "4e", 'o': "4f", 'p': "50", 'q': "51", 'r': "52", 's': "53", 't': "54", 'u': "55", 'v': "56", 'w': "57", 'x': "58", 'y': "59", 'z': "5a", '{': "5b", '|': "5c", '}': "5d", '~': "5e", 'Ã': "5f", 'Ä': "60", 'Å': "61", 'Æ': "62", 'Ç': "63", 'È': "64", 'É': "65", 'Ê': "66", 'Ë': "67", 'Ì': "68", 'Í': "69", 'Î': "6a", 'Ï': "6b", 'Î': "6c" };
 barcodeout = "";
 for upcpart in upc:
  barcodeout = barcodeout + hextoascii.get(upcpart, '');
 return barcodeout;

def get_code128alt_checksum(upc):
 upc = str(upc);
 if(len(upc) < 4): 
  return False;
 upc = convert_ascii_code128_to_hex_code128(upc);
 return get_code128_checksum(upc);

def get_code128dec_checksum(upc):
 upc = str(upc);
 if(len(upc) < 12): 
  return False;
 if(not re.findall("[0-9]{3}", upc)): 
  return False;
 upc_matches = re.findall("[0-9]{3}", upc);
 il = len(upc_matches);
 i = 0;
 upcout = "";
 while(i < il):
  dectohex = format(int(upc_matches[i]), 'x');
  dectohexzero = str(dectohex).zfill(2);
  if(len(dectohexzero)>2):
   return False;
  upcout = upcout+str(dectohexzero);
  i = i + 1;
 upc = upcout;
 return get_code128_checksum(upc);

'''
// MSI (Modified Plessey)
// Source: http://www.barcodeisland.com/msi.phtml
// Source: http://en.wikipedia.org/wiki/MSI_Barcode
'''
def get_msi_checksum(upc):
 upc = str(upc);
 upc = upc.upper();
 upc_matches = list(upc);
 upc_matches = [int(x) for x in upc_matches];
 if(len(upc) % 2==0):
  upc_matches1 = "".join([str(i) for i in upc_matches[1:][::2]]);
  upc_matches1 = list(str(int(upc_matches1) * 2));
  upc_matches1 = [int(x) for x in upc_matches1];
  upc_matches2 = upc_matches[0:][::2];
 else:
  upc_matches1 = "".join([str(i) for i in upc_matches[0:][::2]]);
  upc_matches1 = list(str(int(upc_matches1) * 2));
  upc_matches1 = [int(x) for x in upc_matches1];
  upc_matches2 = upc_matches[1:][::2];
 PreCount = 0;
 UPC_Sum = 0;
 while (PreCount<=len(upc_matches1)-1):
  UPC_Sum += upc_matches1[PreCount];
  PreCount += 1;
 PreCount = 0;
 while (PreCount<=len(upc_matches2)-1):
  UPC_Sum += upc_matches2[PreCount];
  PreCount += 1;
 CheckSum = 10 - (UPC_Sum % 10);
 return str(CheckSum);

'''
// ISSN (International Standard Serial Number)
// Source: http://en.wikipedia.org/wiki/International_Standard_Serial_Number
'''
def validate_issn8_checksum(upc, return_check=False):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>8):
  fix_matches = re.findall("^(\d{8})", upc); 
  fix_matches = fix_matches[0];
  upc = fix_matches[0]+fix_matches[1];
 if(len(upc)>8 or len(upc)<7):
  return False;
 if(len(upc)==7):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc);
 if(len(upc)==8):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc);
 upc_matches = upc_matches[0];
 upc_matches = [int(x) for x in upc_matches];
 AllSum = (upc_matches[0] * 8) + (upc_matches[1] * 7) + (upc_matches[2] * 6) + (upc_matches[3] * 5) + (upc_matches[4] * 4) + (upc_matches[5] * 3) + (upc_matches[6] * 2);
 CheckSum = AllSum % 11;
 if(CheckSum>0):
  CheckSum = 11 - CheckSum;
 if(not return_check and len(upc)==8):
  if(CheckSum!=upc_matches[7]):
   return False;
  if(CheckSum==upc_matches[7]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==7):
  return str(CheckSum);
def get_issn8_checksum(upc):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 return validate_issn8_checksum(upc,True);
def fix_issn8_checksum(upc):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>7):
  fix_matches = re.findall("^(\d{7})", upc); 
  upc = fix_matches[0];
 return upc+str(get_issn8_checksum(upc));
def validate_issn13_checksum(upc, return_check=False):
 upc = str(upc);
 if(not re.findall("^977(\d{9})", upc)):
  return False;
 if(re.findall("^977(\d{9})", upc)):
  return validate_ean13_checksum(upc,return_check);
def get_issn13_checksum(upc):
 upc = str(upc);
 return validate_issn13_checksum(upc,True);
def fix_issn13_checksum(upc):
 upc = str(upc);
 if(not re.findall("^977(\d{9})", upc)):
  return False;
 if(re.findall("^977(\d{9})", upc)):
  return fix_ean13_checksum(upc);

'''
// ISBN (International Standard Book Number)
// Source: http://en.wikipedia.org/wiki/ISBN
'''
def validate_isbn10_checksum(upc, return_check=False):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>10):
  fix_matches = re.findall("^(\d{9})(\d{1}|X{1})", upc); 
  fix_matches = fix_matches[0];
  upc = fix_matches[0]+fix_matches[1];
 if(len(upc)>10 or len(upc)<9):
  return False;
 if(len(upc)==9):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc);
 if(len(upc)==10):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1}|X{1})", upc);
 upc_matches = upc_matches[0];
 upc_matches = [int(x) for x in upc_matches];
 AllSum = (upc_matches[0] * 10) + (upc_matches[1] * 9) + (upc_matches[2] * 8) + (upc_matches[3] * 7) + (upc_matches[4] * 6) + (upc_matches[5] * 5) + (upc_matches[6] * 4) + (upc_matches[7] * 3) + (upc_matches[8] * 2);
 CheckSum = 0;
 while((AllSum + (CheckSum * 1)) % 11):
  CheckSum += 1;
 if(CheckSum==10):
  CheckSum = "X";
 if(not return_check and len(upc)==10):
  if(str(CheckSum)!=upc_matches[9]):
   return False;
  if(str(CheckSum)==upc_matches[9]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==9):
  return str(CheckSum);
def get_isbn10_checksum(upc):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 return validate_isbn10_checksum(upc,True);
def fix_isbn10_checksum(upc):
 upc = str(upc);
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>9):
  fix_matches = re.findall("^(\d{9})", upc);
  upc = fix_matches[1];
 return upc+str(get_isbn10_checksum(upc));
def validate_isbn13_checksum(upc, return_check=False):
 upc = str(upc);
 if(not re.findall("^(97[8-9])(\d{9})", upc)):
  return False;
 if(re.findall("^(97[8-9])(\d{9})", upc)):
  return validate_ean13_checksum(upc,return_check);
def get_isbn13_checksum(upc):
 upc = str(upc);
 return validate_isbn13_checksum(upc,True);
def fix_isbn13_checksum(upc):
 upc = str(upc);
 if(not re.findall("^(97[8-9])(\d{9})", upc)):
  return False;
 if(re.findall("^(97[8-9])(\d{9})", upc)):
  return fix_ean13_checksum(upc);

'''
// ISMN (International Standard Music Number)
// Source: http://en.wikipedia.org/wiki/International_Standard_Music_Number
// Source: http://www.ismn-international.org/whatis.html
// Source: http://www.ismn-international.org/manual_1998/chapter2.html
'''
def validate_ismn10_checksum(upc, return_check=False):
 upc = str(upc);
 upc = upc.replace("M", "");
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>9):
  fix_matches = re.findall("^(\d{8})(\d{1})", upc); 
  fix_matches = fix_matches[0];
  upc = fix_matches[0]+fix_matches[1];
 if(len(upc)>9 or len(upc)<8):
  return False;
 if(len(upc)==8):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc);
 if(len(upc)==9):
  upc_matches = re.findall("^(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})(\d{1})", upc);
 upc_matches = upc_matches[0];
 upc_matches = [int(x) for x in upc_matches];
 AllSum = (3 * 3) + (upc_matches[0] * 1) + (upc_matches[1] * 3) + (upc_matches[2] * 1) + (upc_matches[3] * 3) + (upc_matches[4] * 1) + (upc_matches[5] * 3) + (upc_matches[6] * 1) + (upc_matches[7] * 3);
 CheckSum = 1;
 while((AllSum + (CheckSum * 1)) % 10):
  CheckSum += 1;
 if(not return_check and len(upc)==9):
  if(CheckSum!=upc_matches[8]):
   return False;
  if(CheckSum==upc_matches[8]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(upc)==8):
  return str(CheckSum);
def get_ismn10_checksum(upc):
 upc = str(upc);
 upc = upc.replace("M", "");
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 return validate_ismn10_checksum(upc,True);
def fix_ismn10_checksum(upc):
 upc = str(upc);
 upc = upc.replace("M", "");
 upc = upc.replace("-", "");
 upc = upc.replace(" ", "");
 if(len(upc)>9):
  fix_matches = re.findall("^(\d{9})", upc); 
  upc = fix_matches[1];
 return upc+str(get_ismn10_checksum(upc));
def validate_ismn13_checksum(upc, return_check=False):
 upc = str(upc);
 if(not re.findall("^9790(\d{8})", upc)):
  return False;
 if(re.findall("^9790(\d{8})", upc)):
  return validate_ean13_checksum(upc,return_check);
def get_ismn13_checksum(upc):
 upc = str(upc);
 return validate_ismn13_checksum(upc,True);
def fix_ismn13_checksum(upc):
 upc = str(upc);
 if(not re.findall("^9790(\d{8})", upc)):
  return False;
 if(re.findall("^9790(\d{8})", upc)):
  return fix_ean13_checksum(upc);

'''
// Get variable weight price checksum
// Source: http://wiki.answers.com/Q/How_does_a_price_embedded_bar_code_work
// Source: http://en.wikipedia.org/wiki/Universal_Product_Code#Prefixes
// Source: http://barcodes.gs1us.org/GS1%20US%20BarCodes%20and%20eCom%20-%20The%20Global%20Language%20of%20Business.htm
'''
def validate_vw_price_checksum(price, return_check=False):
 price = str(price);
 if(len(price)==1):
  price = "000"+price;
 if(len(price)==2):
  price = "00"+price;
 if(len(price)==3):
  price = "0"+price;
 if(len(price)>5):
  if(re.findall("^(\d{5})", price)):
   price_matches = re.findall("^(\d{5})", price);
   price = price_matches[0];
 price_split = list(price);
 price_split = [int(x) for x in price_split];
 numrep1 = [0, 2, 4, 6, 8, 9, 1, 3, 5, 7];
 numrep2 = [0, 3, 6, 9, 2, 5, 8, 1, 4, 7];
 numrep3 = [0, 5, 9, 4, 8, 3, 7, 2, 6, 1];
 if(len(price)==4):
  price_split[0] = numrep1[price_split[0]];
  price_split[1] = numrep1[price_split[1]];
  price_split[2] = numrep2[price_split[2]];
  price_split[3] = numrep3[price_split[3]];
  price_add = (price_split[0] + price_split[1] + price_split[2] + price_split[3]) * 3;
 if(len(price)==5):
  price_split[1] = numrep1[price_split[1]];
  price_split[2] = numrep1[price_split[2]];
  price_split[3] = numrep2[price_split[3]];
  price_split[4] = numrep3[price_split[4]]; 
  price_add = (price_split[1] + price_split[2] + price_split[3] + price_split[4]) * 3;
 CheckSum = price_add % 10;
 if(not return_check and len(price)==5):
  if(CheckSum!=price_split[0]):
   return False;
  if(CheckSum==price_split[0]):
   return True;
 if(return_check):
  return str(CheckSum);
 if(len(price)==4):
  return str(CheckSum);
 return str(CheckSum);
def get_vw_price_checksum(price, return_check=False):
 price = str(price);
 if(len(price)==1):
  price = "000"+price;
 if(len(price)==2):
  price = "00"+price;
 if(len(price)==3):
  price = "0"+price;
 return validate_vw_price_checksum(price,True);
def fix_vw_price_checksum(price):
 price = str(price);
 if(len(price)==1):
  price = "000"+price;
 if(len(price)==2):
  price = "00"+price;
 if(len(price)==3):
  price = "0"+price;
 if(len(price)==5):
  fix_matches = re.findall("^(\d{1})(\d{4})", price); 
  fix_matches = fix_matches[0];
  price = fix_matches[1];
 if(len(price)>4):
  fix_matches = re.findall("^(\d{4})", price); 
  price = fix_matches[0];
 return str(get_vw_price_checksum(price,True))+price;
